#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Copyright 2019 Pintaudi Giorgio

from enum import Enum
from typing import Tuple, Optional, Dict, Union

import wagascianpy.pyrmod.pyrmod


class CCCModes(Enum):
    undefined = 0
    spill_off = 1
    continuous = 2
    internal_spill = 3
    external_spill = 4


# noinspection PyTypeChecker
class CCC(wagascianpy.pyrmod.pyrmod.PyrameSlowModule):
    _module_id = "ccc_control"
    _module_name = "ccc"

    def __init__(self,
                 simulate=False,  # type: bool
                 simulation_dic=None,  # type: Optional[Dict[str, Tuple[int, str]]]
                 mode=CCCModes.undefined,  # type: CCCModes
                 period=260,  # type: int
                 active_time=5000  # type: int
                 ):
        # type: (...) -> None
        """ Create a CCC object

        Args:
            simulate         : True to simulate pyrame calls
            simulation_dic   : Dictionary with the return values of the simulated calls
            mode             : type of CCC operation mode
            period           : period between spills (only internal spill mode)
            active_time      : acquisition window duration (only internal spill mode)

        Returns:
            CCC object
        """

        if mode == CCCModes.undefined:
            conf_string = "ccc(ip=192.168.10.16,spill_type=undefined)"
        elif mode == CCCModes.spill_off:
            conf_string = "ccc(ip=192.168.10.16,spill_type=spill off)"
        elif mode == CCCModes.continuous:
            conf_string = "ccc(ip=192.168.10.16,spill_type=continuous)"
        elif mode == CCCModes.internal_spill:
            conf_string = "ccc(ip=192.168.10.16,spill_type=internal spill," \
                          "period={},active_time={})".format(period, active_time)
        else:
            conf_string = "ccc(ip=192.168.10.16,spill_type=undefined)"

        super(CCC, self).__init__(module_name=self._module_name,
                                  module_id=self._module_id,
                                  conf_string=conf_string,
                                  simulate=simulate,
                                  simulation_dic=simulation_dic)

    def switch_mode(self, mode, period=260, active_time=5000, ):
        # type: (CCCModes, int, int) -> Tuple[int, str]
        """ Create a CCCMode object

            Args:
                mode             : type of spill that CCC generates
                period (ms)      : period of the internal spill (default 260 ms)
                active_time (us) : width of the acquisition gate (default 5 ms)

            Returns:
                CCCMode object
        """
        if mode == CCCModes.undefined:
            mode_string = "undefined"
        elif mode == CCCModes.spill_off:
            mode_string = "spill off"
        elif mode == CCCModes.continuous:
            mode_string = "continuous"
        elif mode == CCCModes.internal_spill:
            mode_string = "internal spill"
        elif mode == CCCModes.external_spill:
            mode_string = "external spill"
        else:
            raise ValueError("mode not recognized : {}".format(mode))

        try:
            retcode, res = self.execcmd("switch_mode", mode_string, period, active_time)
        except RuntimeError as err:
            print("CCC : switch_mode : %s" % str(err))
            return 0, str(err)
        else:
            return retcode, res

    def spill_off(self):
        # type: (...) -> Tuple[int, str]
        """ CCC do not issue any trigger """
        return self.switch_mode(mode=CCCModes.spill_off)

    def spill_continuous(self):
        # type: (...) -> Tuple[int, str]
        """ CCC is triggered by the beam line trigger and issues one beam spill followed by six non beam spills """
        return self.switch_mode(mode=CCCModes.continuous)

    def spill_external(self):
        # type: (...) -> Tuple[int, str]
        """ CCC is triggered by an external trigger and issues only one beam spill """
        return self.switch_mode(mode=CCCModes.external_spill)

    def spill_internal(self, period=260, active_time=5000):
        # type: (int, int) -> Tuple[int, str]
        """ CCC issues internally generated triggers spaced by period with an active time windows of active_time

            Args:
                period (ms)      : period of the internal spill (default 260 ms)
                active_time (us) : width of the acquisition gate (default 5 ms)
        """
        return self.switch_mode(mode=CCCModes.internal_spill, period=period, active_time=active_time)

    def get_spill_type(self):
        # type: (...) -> Tuple[int, str]
        """ Return the current spill type

            Returns:
                tuple (return code = 0 for error or 1 of success, spill type as string)
        """
        try:
            retcode, res = self.execcmd("get_spill_type", "no")
        except RuntimeError as err:
            print("CCC : get_spill_type : %s" % str(err))
            return 0, str(err)
        else:
            return retcode, res

    def get_spill_period(self):
        # type: (...) -> Tuple[int, Union[int, str]]
        """ Return the current spill period

            Returns:
                tuple (return code = 0 for error or 1 of success, spill period)
        """
        try:
            retcode, res = self.execcmd("get_spill_period")
        except RuntimeError as err:
            print("CCC : get_spill_period : %s" % str(err))
            return 0, str(err)
        else:
            return retcode, int(res)

    def get_active_time(self):
        # type: (...) -> Tuple[int, Union[int, str]]
        """Return the current gate window"""
        try:
            retcode, res = self.execcmd("get_active_time")
        except RuntimeError as err:
            print("CCC : get_active_time : %s" % str(err))
            return 0, str(err)
        else:
            return retcode, int(res)



import imp
import os
import unittest

from wagascianpy.utils.utils import silentremovedirs
# noinspection PyProtectedMember
from wagascianpy.viewer.configuration import Configuration, RepositoryType, fill_configuration

viewer_module_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), '..', "scripts", "viewer",
                                  "wagasciviewer.py")
wagasciviewer = imp.load_source("wagasciviewer", viewer_module_path)


class TestCheckers(unittest.TestCase):
    def setUp(self):
        fill_configuration()
        self.test_dir = os.path.abspath('../../WagasciAnpy/test')
        databases_dir = os.path.abspath('../../WagasciAnpy/databases')
        Configuration.wagascidb.override({
            'wagasci_database': os.path.join(databases_dir, 'wagascidb.db'),
            'wagasci_repository': os.path.join(self.test_dir, 'test_viewer'),
            'wagasci_download_location': os.path.join(self.test_dir, 'test_viewer'),
            'wagasci_decoded_location': os.path.join(self.test_dir, 'tmp_decoded'),
            'repository_type': RepositoryType.Simple
        })
        Configuration.bsddb.override({
            'bsd_repository': Configuration.bsddb.bsd_repository(),
            'bsd_database': os.path.join(databases_dir, 'bsddb.db'),
            'bsd_download_location': os.path.join(self.test_dir, 'tmp_bsd_download')
        })
        Configuration.global_configuration.override({
            't2krun': 10,
            'history_location': os.path.join(self.test_dir, 'tmp_adc_calibration')
        })
        Configuration.viewer.override({
            'batch_mode': True,
            'update_wagasci_database': False,
            'rebuild_wagasci_database': False,
            'update_bsd_database': False,
            'rebuild_bsd_database': False,
            'wagasci_libdir': None,
            'only_good_runs': False,
            'include_overlapping': False
        })
        Configuration.run_select.override({
            'get_time_interval': False,
            'get_run_interval': False,
            'get_all': False,
            'start_time': None,
            'stop_time': None,
            'start_run': 92,
            'stop_run': 95
        })
        self.app = wagasciviewer.Application(config=Configuration)
        self.app.get_run_interval(start_run=Configuration.run_select.start_run(),
                                  stop_run=Configuration.run_select.stop_run(),
                                  only_good_runs=Configuration.viewer.only_good_runs())

    def tearDown(self):
        silentremovedirs(os.path.join(self.test_dir, 'tmp_bsd_download'))
        silentremovedirs(os.path.join(self.test_dir, 'tmp_decoded'))
        silentremovedirs(os.path.join(self.test_dir, 'tmp_adc_calibration'))

    def decoder(self):
        Configuration.analyzer_configuration.override({
            'decoder': True,
            'overwrite_flag': True
        })
        self.app.analyze(overwrite_flag=Configuration.analyzer_configuration.overwrite_flag())

    def all_but_peu(self):
        Configuration.analyzer_configuration.override({
            'decoder': True,
            'spill_number_fixer': True,
            'beam_summary_data': True,
            'temperature': True,
            'overwrite_flag': True
        })
        self.app.analyze(overwrite_flag=Configuration.analyzer_configuration.overwrite_flag())

    def test_single_multithread_analyzer(self):
        self.decoder()

    def test_single_singlethread_analyzer(self):
        self.decoder()
        Configuration.analyzer_configuration.override({
            'spill_number_fixer': True,
            'overwrite_flag': True
        })
        self.app.analyze(overwrite_flag=Configuration.analyzer_configuration.overwrite_flag())

    def test_multiple_analyzers(self):
        self.all_but_peu()

    def test_single_multirun_analyzer(self):
        self.all_but_peu()
        Configuration.analyzer_configuration.override({
            'adc_calibration': True,
            'overwrite_flag': True
        })
        self.app.analyze(overwrite_flag=Configuration.analyzer_configuration.overwrite_flag())

    def test_multiple_multirun_analyzers(self):
        Configuration.analyzer_configuration.override({
            'all_analyzers': True,
            'overwrite_flag': True
        })
        self.app.analyze(overwrite_flag=Configuration.analyzer_configuration.overwrite_flag())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio
#

import os
import unittest

import wagascianpy.database.bsddb
import wagascianpy.utils.utils


class TestBsdDatabaseCreation(unittest.TestCase):
    def setUp(self):
        self.current_dir = os.path.dirname(os.path.realpath(__file__))
        self.repository_location = os.path.join(self.current_dir, "test_bsd_repo")
        self.database_location = os.path.join(self.current_dir, "test.db")
        wagascianpy.utils.utils.silentremove(self.database_location)

    def tearDown(self):
        wagascianpy.utils.utils.silentremove(self.database_location)

    def test_local_db_creation(self):
        with wagascianpy.database.bsddb.BsdDataBase(self.database_location, self.repository_location,
                                                    None, 10, False, True) as db:
            self.assertFalse(db.is_fresh_database())

    def test_get_all(self):
        with wagascianpy.database.bsddb.BsdDataBase(self.database_location, self.repository_location,
                                                    None, 10, False, True) as db:
            self.assertEqual(len(db.get_all()), 10)

    def test_pretty_print(self):
        with wagascianpy.database.bsddb.BsdDataBase(self.database_location, self.repository_location,
                                                    None, 10, False, True) as db:
            for record in db.get_all():
                wagascianpy.database.bsddb.BsdRecord(None, record).pretty_print()
                print("")

    def test_get_interval_1(self):
        with wagascianpy.database.bsddb.BsdDataBase(self.database_location, self.repository_location,
                                                    None, 10, False, True) as db:
            # From 1573603898 : Wednesday, November 13, 2019 9:11:38 JST
            # To 1573619355 : Wednesday, November 13, 2019 13:29:15 JST
            for record in db.get_time_interval(1573603898, 1573619355):
                start_time = wagascianpy.database.bsddb.BsdRecord(None, record). \
                    get_start_datetime().strftime('%Y/%m/%d %H:%M:%S %Z')
                stop_time = wagascianpy.database.bsddb.BsdRecord(None, record). \
                    get_stop_datetime().strftime('%Y/%m/%d %H:%M:%S %Z')
                print("Record name = %s" % record["name"])
                print("Start time  = %s" % start_time)
                print("Stop time   = %s" % stop_time)

    def test_get_interval_2(self):
        with wagascianpy.database.bsddb.BsdDataBase(self.database_location, self.repository_location,
                                                    None, 10, False, True) as db:
            # From 1573603898 : Wednesday, November 13, 2019 9:11:38 JST
            # To 1573606539 : Wednesday, November 13, 2019 9:55:39 JST
            record = db.get_time_interval(1573603898, 1573606539)[0]
            start_time = wagascianpy.database.bsddb.BsdRecord(None, record). \
                get_start_datetime().strftime('%Y/%m/%d %H:%M:%S %Z')
            stop_time = wagascianpy.database.bsddb.BsdRecord(None, record). \
                get_stop_datetime().strftime('%Y/%m/%d %H:%M:%S %Z')
            self.assertEqual(record["name"], "bsd_run0830152_00v01.root")
            self.assertEqual(start_time, "2019/11/13 09:11:38 JST")
            self.assertEqual(stop_time, "2019/11/13 09:55:39 JST")

    def test_get_interval_3(self):
        with wagascianpy.database.bsddb.BsdDataBase(self.database_location, self.repository_location,
                                                    None, 10, False, True) as db:
            record = db.get_time_interval("2019/11/13 09:11:38 JST", "2019/11/13 09:55:39 JST")[0]
            start_time = wagascianpy.database.bsddb.BsdRecord(None, record). \
                get_start_datetime().strftime('%Y/%m/%d %H:%M:%S %Z')
            stop_time = wagascianpy.database.bsddb.BsdRecord(None, record). \
                get_stop_datetime().strftime('%Y/%m/%d %H:%M:%S %Z')
            self.assertEqual(record["name"], "bsd_run0830152_00v01.root")
            self.assertEqual(start_time, "2019/11/13 09:11:38 JST")
            self.assertEqual(stop_time, "2019/11/13 09:55:39 JST")


class TestBsdDatabaseAccess(unittest.TestCase):
    def setUp(self):
        self.current_dir = os.path.dirname(os.path.realpath(__file__))
        self.local_repository_location = os.path.join(self.current_dir, 'test_bsd_repo')
        self.local_database_location = os.path.join(self.current_dir, "test.db")
        self.remote_repository_location = 'kekcc:/hsm/nu/wagasci/data/bsd/test'
        self.remote_database_location = 'kekcc:/hsm/nu/wagasci/data/bsd/test/test.db'
        self.download_location = os.path.join(self.current_dir, "test_download")
        wagascianpy.utils.utils.silentremovedirs(self.download_location)
        wagascianpy.utils.utils.silentremove(self.local_database_location)
        wagascianpy.utils.utils.mkdir_p(self.download_location)

    def tearDown(self):
        wagascianpy.utils.utils.silentremove(self.local_database_location)
        wagascianpy.utils.utils.silentremovedirs(self.download_location)

    def test_local_repository(self):
        with wagascianpy.database.bsddb.BsdDataBase(bsd_database_location=self.local_database_location,
                                                    bsd_repository_location=self.local_repository_location,
                                                    bsd_repository_download_location=None,
                                                    t2kruns=10,
                                                    update_db=False,
                                                    rebuild_db=True) as db:
            self.assertEqual(len(db.get_all()), 10)

    def test_remote_repository(self):
        with wagascianpy.database.bsddb.BsdDataBase(bsd_repository_location=self.remote_repository_location,
                                                    bsd_database_location=self.local_database_location,
                                                    bsd_repository_download_location=self.download_location,
                                                    t2kruns=10,
                                                    update_db=False,
                                                    rebuild_db=True) as db:
            self.assertEqual(len(db.get_all()), 10)

    def test_remote_database(self):
        with wagascianpy.database.bsddb.BsdDataBase(bsd_repository_location=self.remote_repository_location,
                                                    bsd_database_location=self.remote_database_location,
                                                    bsd_repository_download_location=self.download_location,
                                                    t2kruns=10,
                                                    update_db=False,
                                                    rebuild_db=True) as db:
            self.assertEqual(len(db.get_all()), 10)


if __name__ == '__main__':
    unittest.main()



import unittest
import numpy.testing

from wagascianpy.analysis.spill import *


class TestWagasciSpillCreation(unittest.TestCase):
    def test_wagasci_spill_constructor(self):
        WagasciSpill()

    def test_wagasci_spill_setters(self):
        spill = WagasciSpill()
        spill.converted_spill_number = 1
        spill.good_spill_flag = True
        spill.spill_mode = WAGASCI_SPILL_BEAM_MODE
        spill.spill_count = 1
        spill.spill_number = 1
        spill.temperature = 1.
        spill.humidity = 1.
        spill.wagasci_run = 1

        self.assertEqual(spill.converted_spill_number, 1)
        self.assertEqual(spill.good_spill_flag, True)
        self.assertEqual(spill.spill_mode, WAGASCI_SPILL_BEAM_MODE)
        self.assertEqual(spill.spill_count, 1)
        self.assertEqual(spill.spill_number, 1)
        self.assertAlmostEqual(spill.temperature, 1, delta=0.0001)
        self.assertAlmostEqual(spill.humidity, 1, delta=0.0001)
        self.assertEqual(spill.wagasci_run, 1)

        self.assertIsInstance(spill.converted_spill_number, int)
        self.assertIsInstance(spill.good_spill_flag, bool)
        self.assertIsInstance(spill.spill_mode, int)
        self.assertIsInstance(spill.spill_count, int)
        self.assertIsInstance(spill.spill_number, int)
        self.assertIsInstance(spill.temperature, float)
        self.assertIsInstance(spill.humidity, float)
        self.assertIsInstance(spill.wagasci_run, int)


class TestWagasciSpillMethods(unittest.TestCase):
    def setUp(self):
        self.spill = WagasciSpill()
        self.spill.converted_spill_number = 1
        self.spill.good_spill_flag = True
        self.spill.spill_mode = WAGASCI_SPILL_BEAM_MODE
        self.spill.spill_count = 1
        self.spill.spill_number = 1
        self.spill.temperature = 1.
        self.spill.humidity = 1.
        self.spill.wagasci_run = 1

    def test_are_all_defined(self):
        self.assertTrue(self.spill.are_all_defined())

    def test_get_type_char(self):
        self.assertEqual(self.spill._get_type_char("converted_spill_number"), 'i')
        self.assertEqual(self.spill._get_type_char("good_spill_flag"), 'o')
        self.assertEqual(self.spill._get_type_char("spill_mode"), 'i')
        self.assertEqual(self.spill._get_type_char("spill_count"), 'i')
        self.assertEqual(self.spill._get_type_char("spill_number"), 'i')
        self.assertEqual(self.spill._get_type_char("temperature"), 'd')
        self.assertEqual(self.spill._get_type_char("humidity"), 'd')
        self.assertEqual(self.spill._get_type_char("wagasci_run"), 'i')

    def test_get_numpy_type(self):
        self.assertEqual(self.spill._get_numpy_type("converted_spill_number"), numpy.int32)
        self.assertEqual(self.spill._get_numpy_type("good_spill_flag"), numpy.bool)
        self.assertEqual(self.spill._get_numpy_type("spill_mode"), numpy.int32)
        self.assertEqual(self.spill._get_numpy_type("spill_count"), numpy.int32)
        self.assertEqual(self.spill._get_numpy_type("spill_number"), numpy.int32)
        self.assertEqual(self.spill._get_numpy_type("temperature"), numpy.float64)
        self.assertEqual(self.spill._get_numpy_type("humidity"), numpy.float64)
        self.assertEqual(self.spill._get_numpy_type("wagasci_run"), numpy.int32)

    def test_get_type_str(self):
        self.assertEqual(self.spill._get_type_str("converted_spill_number"), 'converted_spill_number/I')
        self.assertEqual(self.spill._get_type_str("good_spill_flag"), 'good_spill_flag/O')
        self.assertEqual(self.spill._get_type_str("spill_mode"), 'spill_mode/I')
        self.assertEqual(self.spill._get_type_str("spill_count"), 'spill_count/I')
        self.assertEqual(self.spill._get_type_str("spill_number"), 'spill_number/I')
        self.assertEqual(self.spill._get_type_str("temperature"), 'temperature/D')
        self.assertEqual(self.spill._get_type_str("humidity"), 'humidity/D')
        self.assertEqual(self.spill._get_type_str("wagasci_run"), 'wagasci_run/I')

    def test_get_array(self):
        self.assertEqual(self.spill._get_array("converted_spill_number"), numpy.array([1], dtype=numpy.int32))
        self.assertEqual(self.spill._get_array("good_spill_flag"), numpy.array([True], dtype=numpy.bool))
        self.assertEqual(self.spill._get_array("spill_mode"), numpy.array([WAGASCI_SPILL_BEAM_MODE], dtype=numpy.int32))
        self.assertEqual(self.spill._get_array("spill_count"), numpy.array([1], dtype=numpy.int32))
        self.assertEqual(self.spill._get_array("spill_number"), numpy.array([1], dtype=numpy.int32))
        self.assertEqual(self.spill._get_array("temperature"), numpy.array([1.], dtype=numpy.float64))
        self.assertEqual(self.spill._get_array("humidity"), numpy.array([1.], dtype=numpy.float64))
        self.assertEqual(self.spill._get_array("wagasci_run"), numpy.array([1], dtype=numpy.int32))

    def test_get_array_list(self):
        expected_array_list = [
            SpillArrayInfo(
                name='spill_count',
                array=numpy.array([1], dtype=numpy.int32),
                type_char='i',
                type_str='spill_count/I'),
            SpillArrayInfo(
                name='spill_number',
                array=numpy.array([1], dtype=numpy.int32),
                type_char='i',
                type_str='spill_number/I'),
            SpillArrayInfo(
                name='converted_spill_number',
                array=numpy.array([1], dtype=numpy.int32),
                type_char='i',
                type_str='converted_spill_number/I'),
            SpillArrayInfo(
                name='spill_mode',
                array=numpy.array([WAGASCI_SPILL_BEAM_MODE], dtype=numpy.int32),
                type_char='i',
                type_str='spill_mode/I'),
            SpillArrayInfo(
                name='good_spill_flag',
                array=numpy.array([True], dtype=numpy.bool),
                type_char='o',
                type_str='good_spill_flag/O'),
            SpillArrayInfo(
                name='temperature',
                array=numpy.array([1.], dtype=numpy.float64),
                type_char='d',
                type_str='temperature/D'),
            SpillArrayInfo(
                name='humidity',
                array=numpy.array([1.], dtype=numpy.float64),
                type_char='d',
                type_str='humidity/D'),
            SpillArrayInfo(
                name='wagasci_run',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='wagasci_run/I'),
        ]
        self.assertEqual(sorted(expected_array_list), sorted(self.spill.get_array_list()))

    def test_set_array_list(self):
        expected_array_list = [
            SpillArrayInfo(
                name='spill_count',
                array=numpy.array([2], dtype=numpy.int32),
                type_char='i',
                type_str='spill_count/I'),
            SpillArrayInfo(
                name='spill_number',
                array=numpy.array([2], dtype=numpy.int32),
                type_char='i',
                type_str='spill_number/I'),
            SpillArrayInfo(
                name='converted_spill_number',
                array=numpy.array([2], dtype=numpy.int32),
                type_char='i',
                type_str='converted_spill_number/I'),
            SpillArrayInfo(
                name='spill_mode',
                array=numpy.array([WAGASCI_SPILL_NO_BEAM_MODE], dtype=numpy.int32),
                type_char='i',
                type_str='spill_mode/I'),
            SpillArrayInfo(
                name='good_spill_flag',
                array=numpy.array([False], dtype=numpy.bool),
                type_char='o',
                type_str='good_spill_flag/O'),
            SpillArrayInfo(
                name='temperature',
                array=numpy.array([2], dtype=numpy.float64),
                type_char='d',
                type_str='temperature/D'),
            SpillArrayInfo(
                name='humidity',
                array=numpy.array([2], dtype=numpy.float64),
                type_char='d',
                type_str='humidity/D'),
            SpillArrayInfo(
                name='wagasci_run',
                array=numpy.array([2], dtype=numpy.float64),
                type_char='i',
                type_str='wagasci_run/I'),
        ]
        self.spill.set_array_list(expected_array_list)
        self.assertEqual(sorted(expected_array_list), sorted(self.spill.get_array_list()))


class TestBsdSpillCreation(unittest.TestCase):
    def test_bsd_spill_constructor(self):
        BsdSpill()

    def test_bsd_spill_setters(self):
        spill = BsdSpill()
        spill.converted_spill_number = 1
        spill.bsd_good_spill_flag = 1
        spill.timestamp = 1
        spill.pot = 1
        spill.bsd_spill_number = 1
        spill.bunch_pot = list(range(0, 8))

        self.assertEqual(spill.converted_spill_number, 1)
        self.assertEqual(spill.bsd_good_spill_flag, 1)
        self.assertEqual(spill.pot, 1)
        self.assertEqual(spill.timestamp, 1)
        self.assertEqual(spill.bsd_spill_number, 1)
        self.assertIsNone(numpy.testing.assert_almost_equal(spill.bunch_pot,
                                                            numpy.array(list(range(0, 8)), dtype=numpy.float64)))

        self.assertIsInstance(spill.converted_spill_number, int)
        self.assertIsInstance(spill.bsd_good_spill_flag, int)
        self.assertIsInstance(spill.pot, float)
        self.assertIsInstance(spill.timestamp, float)
        self.assertIsInstance(spill.bsd_spill_number, int)
        self.assertIsInstance(spill.bunch_pot, numpy.ndarray)


class TestBsdSpillMethods(unittest.TestCase):
    def setUp(self):
        self.spill = BsdSpill()
        self.spill.converted_spill_number = 1
        self.spill.bsd_good_spill_flag = 1
        self.spill.timestamp = 1
        self.spill.pot = 1
        self.spill.bsd_spill_number = 1
        self.spill.t2k_run = 1
        self.spill.main_ring_run = 1
        self.spill.neutrino_daq_run = 1
        self.spill.horn_current = 1
        self.spill.neutrino_mode = 1
        self.spill.bunch_pot = list(range(0, 8))

    def test_are_all_defined(self):
        self.assertTrue(self.spill.are_all_defined())

    def test_get_type_char(self):
        self.assertEqual(self.spill._get_type_char("converted_spill_number"), 'i')
        self.assertEqual(self.spill._get_type_char("bsd_good_spill_flag"), 'i')
        self.assertEqual(self.spill._get_type_char("pot"), 'd')
        self.assertEqual(self.spill._get_type_char("timestamp"), 'd')
        self.assertEqual(self.spill._get_type_char("bsd_spill_number"), 'i')
        self.assertEqual(self.spill._get_type_char("bunch_pot"), 'd')

    def test_get_numpy_type(self):
        self.assertEqual(self.spill._get_numpy_type("converted_spill_number"), numpy.int32)
        self.assertEqual(self.spill._get_numpy_type("bsd_good_spill_flag"), numpy.int32)
        self.assertEqual(self.spill._get_numpy_type("pot"), numpy.float64)
        self.assertEqual(self.spill._get_numpy_type("timestamp"), numpy.float64)
        self.assertEqual(self.spill._get_numpy_type("bsd_spill_number"), numpy.int32)
        self.assertEqual(self.spill._get_numpy_type("bunch_pot"), numpy.float64)

    def test_get_type_str(self):
        self.assertEqual(self.spill._get_type_str("converted_spill_number"), 'converted_spill_number/I')
        self.assertEqual(self.spill._get_type_str("bsd_good_spill_flag"), 'bsd_good_spill_flag/I')
        self.assertEqual(self.spill._get_type_str("pot"), 'pot/D')
        self.assertEqual(self.spill._get_type_str("timestamp"), 'timestamp/D')
        self.assertEqual(self.spill._get_type_str("bsd_spill_number"), 'bsd_spill_number/I')
        self.assertEqual(self.spill._get_type_str("bunch_pot"), 'bunch_pot[8]/D')

    def test_get_array(self):
        self.assertEqual(self.spill._get_array("converted_spill_number"), numpy.array([1], dtype=numpy.int32))
        self.assertEqual(self.spill._get_array("bsd_good_spill_flag"), numpy.array([1], dtype=numpy.int32))
        self.assertEqual(self.spill._get_array("pot"), numpy.array([1], dtype=numpy.float64))
        self.assertEqual(self.spill._get_array("timestamp"), numpy.array([1], dtype=numpy.float64))
        self.assertEqual(self.spill._get_array("bsd_spill_number"), numpy.array([1], dtype=numpy.int32))
        self.assertIsNone(numpy.testing.assert_almost_equal(self.spill._get_array("bunch_pot"),
                                                            numpy.array(list(range(0, 8)), dtype=numpy.float64)))

    def test_get_array_list(self):
        expected_array_list = [
            SpillArrayInfo(
                name='bsd_spill_number',
                array=numpy.array([1], dtype=numpy.int32),
                type_char='i',
                type_str='bsd_spill_number/I'),
            SpillArrayInfo(
                name='converted_spill_number',
                array=numpy.array([1], dtype=numpy.int32),
                type_char='i',
                type_str='converted_spill_number/I'),
            SpillArrayInfo(
                name='pot',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='d',
                type_str='pot/D'),
            SpillArrayInfo(
                name='timestamp',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='d',
                type_str='timestamp/D'),
            SpillArrayInfo(
                name='bsd_good_spill_flag',
                array=numpy.array([1], dtype=numpy.int32),
                type_char='i',
                type_str='bsd_good_spill_flag/I'),
            SpillArrayInfo(
                name='t2k_run',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='t2k_run/I'),
            SpillArrayInfo(
                name='main_ring_run',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='main_ring_run/I'),
            SpillArrayInfo(
                name='neutrino_daq_run',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='neutrino_daq_run/I'),
            SpillArrayInfo(
                name='horn_current',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='d',
                type_str='horn_current/D'),
            SpillArrayInfo(
                name='neutrino_mode',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='neutrino_mode/I'),
            SpillArrayInfo(
                name='bunch_pot',
                array=numpy.array(list(range(0, 8)), dtype=numpy.float64),
                type_char='d',
                type_str='bunch_pot[8]/D')
        ]
        sorted_array_list = [v for v in sorted(expected_array_list) if v.array.size == 1]
        for i, v in enumerate([v for v in sorted(self.spill.get_array_list()) if v.array.size == 1]):
            self.assertEqual(v, sorted_array_list[i])
        sorted_array_list = [v for v in sorted(expected_array_list) if v.array.size > 1]
        for i, v in enumerate([v for v in sorted(self.spill.get_array_list()) if v.array.size > 1]):
            self.assertIsNone(numpy.testing.assert_almost_equal(v.array, sorted_array_list[i].array))

    def test_set_array_list(self):
        expected_array_list = [
            SpillArrayInfo(
                name='bsd_spill_number',
                array=numpy.array([2], dtype=numpy.int32),
                type_char='i',
                type_str='bsd_spill_number/I'),
            SpillArrayInfo(
                name='converted_spill_number',
                array=numpy.array([2], dtype=numpy.int32),
                type_char='i',
                type_str='converted_spill_number/I'),
            SpillArrayInfo(
                name='pot',
                array=numpy.array([2], dtype=numpy.float64),
                type_char='d',
                type_str='pot/D'),
            SpillArrayInfo(
                name='timestamp',
                array=numpy.array([2], dtype=numpy.float64),
                type_char='d',
                type_str='timestamp/D'),
            SpillArrayInfo(
                name='bsd_good_spill_flag',
                array=numpy.array([2], dtype=numpy.int32),
                type_char='i',
                type_str='bsd_good_spill_flag/I'),
            SpillArrayInfo(
                name='t2k_run',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='t2k_run/I'),
            SpillArrayInfo(
                name='main_ring_run',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='main_ring_run/I'),
            SpillArrayInfo(
                name='neutrino_daq_run',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='neutrino_daq_run/I'),
            SpillArrayInfo(
                name='horn_current',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='d',
                type_str='horn_current/D'),
            SpillArrayInfo(
                name='neutrino_mode',
                array=numpy.array([1], dtype=numpy.float64),
                type_char='i',
                type_str='neutrino_mode/I'),
            SpillArrayInfo(
                name='bunch_pot',
                array=numpy.ones(8, dtype=numpy.float64),
                type_char='d',
                type_str='bunch_pot[8]/D')
        ]
        self.spill.set_array_list(expected_array_list)
        sorted_array_list = [v for v in sorted(expected_array_list) if v.array.size == 1]
        for i, v in enumerate([v for v in sorted(self.spill.get_array_list()) if v.array.size == 1]):
            self.assertEqual(v, sorted_array_list[i])
        sorted_array_list = [v for v in sorted(expected_array_list) if v.array.size > 1]
        for i, v in enumerate([v for v in sorted(self.spill.get_array_list()) if v.array.size > 1]):
            self.assertIsNone(numpy.testing.assert_almost_equal(v.array, sorted_array_list[i].array))


if __name__ == '__main__':
    unittest.main()



import unittest

import wagascianpy.plotting.parse_args
from wagascianpy.plotting.configuration import fill_configuration, Configuration


class TestParseArgs(unittest.TestCase):

    def setUp(self):
        self.args = [
            '--output-string=buyaka1',
            '--output-path=buyaka2',
            '--wagasci-database=buyaka3',
            '--wagasci-decoded-location=buyaka4',
            '--bsd-database=buyaka5',
            '--bsd-repository=buyaka6',
            '--bsd-download-location=buyaka7',
            '--history-location=buyaka8',
            '--t2krun=10',
            '--start-time=today',
            '--stop-time=tomorrow',
            '--start-run=1',
            '--stop-run=2',
            '--delivered-pot',
            '--accumulated-pot',
            '--bsd-spill-history',
            '--wagasci-spill-history',
            '--wagasci-spill-number',
            '--wagasci-fixed-spill',
            '--temperature',
            '--humidity',
            '--gain-history',
            '--dark-noise-history',
            '--threshold-history',
            '--bcid-history',
            '--all',
            '--run-markers',
            '--maintenance-markers',
            '--trouble-markers',
            '--bsd-markers',
            '--save-tfile',
            '--topology=wagasci_top_downstream',
            '--only-good',
            '--dif=17',
            '--chip=17',
            '--channel=17'
        ]

        self.parsed_args = {
            'output_string': 'buyaka1',
            'output_path': 'buyaka2',
            'wagasci_database': 'buyaka3',
            'wagasci_decoded_location': 'buyaka4',
            'bsd_database': 'buyaka5',
            'bsd_repository': 'buyaka6',
            'bsd_download_location': 'buyaka7',
            'history_location': 'buyaka8',
            't2krun': 10,
            'start_time': 'today',
            'stop_time': 'tomorrow',
            'start_run': 1,
            'stop_run': 2,
            'delivered_pot': True,
            'accumulated_pot': True,
            'bsd_spill_history': True,
            'wagasci_spill_history': True,
            'wagasci_spill_number': True,
            'wagasci_fixed_spill': True,
            'temperature': True,
            'humidity': True,
            'gain_history': True,
            'dark_noise_history': True,
            'threshold_history': True,
            'bcid_history': True,
            'all': True,
            'run_markers': True,
            'maintenance_markers': True,
            'trouble_markers': True,
            'bsd_markers': True,
            'save_tfile': True,
            'topology': 'wagasci_top_downstream',
            'only_good': True,
            'dif': 17,
            'chip': 17,
            'channel': 17
        }

    def test_do_not_crash(self):
        wagascianpy.plotting.parse_args.parse_args(self.args)

    def test_list_flattening(self):
        parsed_args = wagascianpy.plotting.parse_args.parse_args(self.args)
        for name, value in vars(parsed_args).items():
            self.assertNotIsInstance(value, list)

    def test_correct_parsing(self):
        parsed_args = wagascianpy.plotting.parse_args.parse_args(self.args)
        for name, value in self.parsed_args.items():
            print("Testing if argument {} = {}".format(name, value))
            self.assertEqual(value, getattr(parsed_args, name))

    def test_change_configuration(self):
        fill_configuration(self.parsed_args)
        # WAGASCI database configuration
        self.assertEqual(self.parsed_args['wagasci_database'],
                         Configuration.wagascidb.wagasci_database())
        self.assertEqual(self.parsed_args['wagasci_decoded_location'],
                         Configuration.wagascidb.wagasci_decoded_location())
        # BSD database configuration
        self.assertEqual(self.parsed_args['bsd_database'],
                         Configuration.bsddb.bsd_database())
        self.assertEqual(self.parsed_args['bsd_download_location'],
                         Configuration.bsddb.bsd_download_location())
        # global configuration
        self.assertEqual(self.parsed_args['t2krun'],
                         Configuration.global_configuration.t2krun())
        self.assertEqual(self.parsed_args['history_location'],
                         Configuration.global_configuration.history_location())

    def test_parse_topology_by_detector(self):
        topology = wagascianpy.plotting.parse_args.parse_plotting_topology("wagasci_downstream")
        self.assertFalse(topology.are_all_enabled())
        self.assertTrue(topology.wagasci_downstream.is_enabled())
        self.assertFalse(topology.wagasci_upstream.is_enabled())
        self.assertFalse(topology.wallmrd_south.is_enabled())
        self.assertFalse(topology.wallmrd_north.is_enabled())

    def test_parse_topology_by_dif(self):
        topology = wagascianpy.plotting.parse_args.parse_plotting_topology("wagasci_downstream_top,"
                                                                           "wallmrd_south_bottom")
        self.assertFalse(topology.are_all_enabled())
        self.assertTrue(topology.wagasci_downstream_top.is_enabled())
        self.assertTrue(topology.wallmrd_south_bottom.is_enabled())
        self.assertFalse(topology.wagasci_upstream_top.is_enabled())
        self.assertFalse(topology.wallmrd_south_top.is_enabled())
        self.assertFalse(topology.wallmrd_north_top.is_enabled())
        self.assertFalse(topology.wagasci_downstream_side.is_enabled())
        self.assertFalse(topology.wagasci_upstream_side.is_enabled())
        self.assertFalse(topology.wallmrd_north_bottom.is_enabled())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio

import unittest

import numpy

import wagascianpy.plotting.colors
import wagascianpy.plotting.marker

import ROOT
ROOT.PyConfig.IgnoreCommandLineOptions = True


class TestMarkers(unittest.TestCase):

    def setUp(self):
        self.canvas = ROOT.TCanvas("c1", "c1", 1280, 720)
        self.graph = ROOT.TGraph(3, numpy.array([0, 1, 2], dtype=numpy.float64),
                                 numpy.array([0, 1, 2], dtype=numpy.float64))

    def test_single_marker(self):
        self.marker = wagascianpy.plotting.marker.Marker(1, "Hello World!")
        self.marker.line_color = wagascianpy.plotting.colors.Colors.Red
        self.output_filename = "./test_single_markers.png"

    def test_double_marker(self):
        self.marker = wagascianpy.plotting.marker.DoubleMarker(left_position=1,
                                                               left_text="Left text",
                                                               right_position=1.5,
                                                               right_text="Right text")
        self.marker.line_color = wagascianpy.plotting.colors.Colors.Blue
        self.marker.fill_color = wagascianpy.plotting.colors.Colors.Orange
        self.marker.transparency = 0.5
        self.output_filename = "./test_double_markers.pdf"

    def tearDown(self):
        self.graph.Draw()
        markers = self.marker.make_tobjects()
        for marker in markers:
            marker.Draw()
        self.canvas.Print(self.output_filename)


if __name__ == '__main__':
    unittest.main()



import sys
import time
import unittest
from enum import Enum

import pyfakefs.fake_filesystem_unittest
from six import string_types

from wagascianpy.utils.classproperty import classproperty
from wagascianpy.utils.configuration import WagasciConfigParser, VirtualConfiguration, conf_setter, conf_getter, \
    Configuration, create_sections


class TestEnum(Enum):
    Foo = 1
    Bar = 2


# noinspection PyMethodParameters,PyPropertyDefinition
class CustomVolatileConfiguration(VirtualConfiguration):
    _test_str = 'test_volatile'

    @classmethod
    def reset(cls):
        cls._test_str = 'test_volatile'

    @classmethod
    def is_volatile(cls):
        return True

    @classproperty
    def test_str(cls):
        return cls._test_str

    @test_str.setter
    def test_str(cls, value):
        conf_setter(cls=cls, name='_test_str', value=value)


# noinspection PyMethodParameters,PyPropertyDefinition
class CustomConfiguration(VirtualConfiguration):
    _test_str = 'test'
    _test_int = 1
    _test_float = 3.14
    _test_bool = False
    _test_enum = TestEnum.Foo
    _test_none_str = ''
    _test_none_int = 0
    _test_none_float = 0.

    @classmethod
    def reset(cls):
        cls._test_str = 'test'
        cls._test_int = 1
        cls._test_float = 3.14
        cls._test_bool = False
        cls._test_enum = TestEnum.Foo
        cls._test_none_str = ''
        cls._test_none_int = 0
        cls._test_none_float = 0.

    @classmethod
    def is_volatile(cls):
        return False

    @classproperty
    def test_str(cls):
        return cls._test_str

    @test_str.setter
    def test_str(cls, value):
        conf_setter(cls=cls, name='_test_str', value=value)

    @classproperty
    def test_int(cls):
        return cls._test_int

    @test_int.setter
    def test_int(cls, value):
        conf_setter(cls=cls, name='_test_int', value=value)

    @classproperty
    def test_float(cls):
        return cls._test_float

    @test_float.setter
    def test_float(cls, value):
        conf_setter(cls=cls, name='_test_float', value=value)

    @classproperty
    def test_bool(cls):
        return cls._test_bool

    @test_bool.setter
    def test_bool(cls, value):
        conf_setter(cls=cls, name='_test_bool', value=value)

    @classproperty
    def test_enum(cls):
        return cls._test_enum

    @test_enum.setter
    def test_enum(cls, value):
        conf_setter(cls=cls, name='_test_enum', value=value)

    @classproperty
    def test_none_str(cls):
        return cls._test_none_str

    @test_none_str.setter
    def test_none_str(cls, value):
        conf_setter(cls=cls, name='_test_none_str', value=value)

    @classproperty
    def test_none_int(cls):
        return cls._test_none_int

    @test_none_int.setter
    def test_none_int(cls, value):
        conf_setter(cls=cls, name='_test_none_int', value=value)

    @classproperty
    def test_none_float(cls):
        return cls._test_none_float

    @test_none_float.setter
    def test_none_float(cls, value):
        conf_setter(cls=cls, name='_test_none_float', value=value)


class CustomConfigParser1(WagasciConfigParser, CustomConfiguration, CustomVolatileConfiguration):
    pass


# noinspection PyUnresolvedReferences,PyPropertyAccess
class TestDefaultParser(unittest.TestCase):

    def setUp(self):
        self.default_parser = WagasciConfigParser()
        self.custom_parser = CustomConfigParser1()

    def tearDown(self):
        self.custom_parser.custom_configuration.reset()

    def test_property_value(self):
        self.assertEqual(CustomConfiguration.test_str, self.custom_parser.custom_configuration.test_str)
        self.assertEqual(CustomConfiguration.test_int, self.custom_parser.custom_configuration.test_int)
        self.assertEqual(CustomConfiguration.test_float, self.custom_parser.custom_configuration.test_float)
        self.assertEqual(CustomConfiguration.test_bool, self.custom_parser.custom_configuration.test_bool)
        self.assertEqual(CustomConfiguration.test_enum, self.custom_parser.custom_configuration.test_enum)

    def test_propery_type(self):
        self.assertIsInstance(self.custom_parser.custom_configuration.test_str, string_types)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_int, int)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_float, float)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_bool, bool)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_enum, TestEnum)

    def test_bool_propery_setter(self):
        self.custom_parser.custom_configuration.test_bool = 'True'
        self.assertTrue(self.custom_parser.custom_configuration.test_bool)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_bool, bool)
        self.custom_parser.custom_configuration.test_bool = 'False'
        self.assertFalse(self.custom_parser.custom_configuration.test_bool)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_bool, bool)

    def test_int_propery_setter(self):
        self.custom_parser.custom_configuration.test_int = '42'
        self.assertEqual(42, self.custom_parser.custom_configuration.test_int)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_int, int)
        self.custom_parser.custom_configuration.test_int = 42.1
        self.assertEqual(42, self.custom_parser.custom_configuration.test_int)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_int, int)

    def test_float_propery_setter(self):
        self.custom_parser.custom_configuration.test_float = '3.14'
        self.assertEqual(3.14, self.custom_parser.custom_configuration.test_float)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_float, float)
        self.custom_parser.custom_configuration.test_float = 3
        self.assertEqual(3., self.custom_parser.custom_configuration.test_float)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_float, float)

    def test_enum_propery_setter(self):
        self.custom_parser.custom_configuration.test_enum = 1
        self.assertEqual(TestEnum.Foo, self.custom_parser.custom_configuration.test_enum)
        self.custom_parser.custom_configuration.test_enum = 'bar'
        self.assertEqual(TestEnum.Bar, self.custom_parser.custom_configuration.test_enum)

    def test_none_str_propery_setter(self):
        self.custom_parser.custom_configuration.test_none_str = None
        self.assertEqual('', self.custom_parser.custom_configuration.test_none_str)

    def test_none_int_propery_setter(self):
        self.custom_parser.custom_configuration.test_none_int = None
        self.assertEqual(0, self.custom_parser.custom_configuration.test_none_int)

    def test_none_float_propery_setter(self):
        self.custom_parser.custom_configuration.test_none_float = None
        self.assertEqual(0.0, self.custom_parser.custom_configuration.test_none_float)

    def test_setter_exceptions(self):
        with self.assertRaises(ValueError):
            self.custom_parser.custom_configuration.test_float = 'aregareg'
        with self.assertRaises(ValueError):
            self.custom_parser.custom_configuration.test_int = 'aregareg'
        with self.assertRaises(ValueError):
            self.custom_parser.custom_configuration.test_bool = 'aregareg'
        with self.assertRaises(ValueError):
            self.custom_parser.custom_configuration.test_enum = 'aregareg'

    def test_get_section(self):
        self.assertEqual(self.custom_parser.get_section('custom_configuration'), CustomConfiguration)
        self.assertEqual(self.custom_parser.custom_configuration.test_str, 'test')
        self.assertRaises(AttributeError, self.default_parser.get_section, 'fdanviounvdafivb')

    def test_get_sections(self):
        self.assertIsInstance(self.default_parser.get_sections(), list)
        self.assertIsInstance(self.custom_parser.get_sections(), list)
        self.assertIn('custom_configuration', self.custom_parser.get_sections())

    def test_prettyprint(self):
        self.custom_parser = CustomConfigParser1()
        self.custom_parser.prettyprint()
        sys.stdout.flush()


###########################################################################################################

# noinspection PyUnresolvedReferences
class TestConfigFile(pyfakefs.fake_filesystem_unittest.TestCase):

    def setUp(self):
        self.setUpPyfakefs()
        self.config_file_path_write = '/tmp/test_write.ini'
        self.config_file_path_read = '/tmp/test_read.ini'

    def test_write_default(self):
        self.custom_parser = CustomConfigParser1()
        self.custom_parser.custom_configuration.test_int = 42
        self.custom_parser.custom_configuration.test_enum = TestEnum.Bar
        self.custom_parser.custom_configuration.test_bool = True
        self.custom_parser.custom_configuration.test_float = 1.62
        self.custom_parser.custom_configuration.test_str = 'abu'
        self.custom_parser.custom_configuration.test_none_str = None
        self.custom_parser.custom_configuration.test_none_int = None
        self.custom_parser.custom_configuration.test_none_float = None
        self.custom_parser.write(self.config_file_path_write)
        with open(self.config_file_path_write, 'r') as fin:
            content = fin.read()
            self.assertIn('test_str = abu', content)
            self.assertIn('test_int = 42', content)
            self.assertIn('test_float = 1.62', content)
            self.assertIn('test_bool = True', content)
            self.assertIn('test_enum = Bar', content)
            self.assertIn('test_none_str = ', content)
            self.assertIn('test_none_int = 0', content)
            self.assertIn('test_none_float = 0.0', content)
        self.custom_parser.custom_configuration.reset()

    def test_read(self):
        with open(self.config_file_path_read, 'w') as fout:
            fout.writelines(['[custom_configuration]\n',
                             'test_str = abu\n',
                             'test_int = 42\n',
                             'test_bool = True\n',
                             'test_float = 1.62\n',
                             'test_enum = Bar'])

        self.custom_parser = CustomConfigParser1(self.config_file_path_read)
        self.assertEqual('abu', self.custom_parser.custom_configuration.test_str)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_str, str)
        self.assertEqual(42, self.custom_parser.custom_configuration.test_int)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_int, int)
        self.assertEqual(1.62, self.custom_parser.custom_configuration.test_float)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_float, float)
        self.assertEqual(True, self.custom_parser.custom_configuration.test_bool)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_bool, bool)
        self.assertEqual(TestEnum.Bar, self.custom_parser.custom_configuration.test_enum)
        self.assertIsInstance(self.custom_parser.custom_configuration.test_enum, TestEnum)
        self.custom_parser.custom_configuration.reset()

    def test_volatile(self):
        self.custom_parser = CustomConfigParser1(self.config_file_path_write)
        with open(self.config_file_path_write, 'r') as fin:
            content = fin.read()
            self.assertNotIn('test_str = test_volatile', content)
        self.assertEqual('test_volatile', self.custom_parser.custom_volatile_configuration.test_str)


###########################################################################################################

# noinspection PyUnresolvedReferences
class TestConfigurationProvider(pyfakefs.fake_filesystem_unittest.TestCase):

    def setUp(self):
        self.setUpPyfakefs()

    def test_default_configuration(self):
        parser = WagasciConfigParser()
        self.assertEqual(parser.global_configuration.t2krun, Configuration.global_configuration.t2krun())

    def test_create_section(self):
        Configuration.create_section("test_section")
        Configuration.test_section.override({'foo': 42})
        self.assertEqual(42, Configuration.test_section.foo())
        Configuration.create_section("test_section")  # should not raise
        Configuration.delete_section("test_section")

    def test_get_sections(self):
        Configuration.create_section("test_section")
        Configuration.test_section.override({'foo': 42})
        self.assertIn("test_section", Configuration.get_sections())
        Configuration.delete_section("test_section")


###########################################################################################################

# noinspection PyMethodParameters,PyPropertyDefinition
class CustomVolatileConfiguration(VirtualConfiguration):
    _test_str = "buyaka"
    _test_bool = True

    @classmethod
    def reset(cls):
        cls._test_str = "buyaka"
        cls._test_bool = True

    @classmethod
    def is_volatile(cls):
        return True

    @classproperty
    def test_str(cls):
        return conf_getter(cls=cls, name='_test_str')

    @test_str.setter
    def test_str(cls, value):
        conf_setter(cls=cls, name='_test_str', value=value)

    @classproperty
    def test_bool(cls):
        return conf_getter(cls=cls, name='_test_bool')

    @test_bool.setter
    def test_bool(cls, value):
        conf_setter(cls=cls, name='_test_bool', value=value)


class CustomConfigParser2(WagasciConfigParser, CustomVolatileConfiguration):
    pass


@create_sections(parser_class=CustomConfigParser2)
class CustomConfigurationProvider(Configuration):
    pass


class TestCustomConfigurationProvider(pyfakefs.fake_filesystem_unittest.TestCase):

    def setUp(self):
        self.setUpPyfakefs()

    def test_default_configuration(self):
        parser = CustomConfigParser2()
        self.assertEqual(parser.custom_volatile_configuration.test_str,
                         CustomConfigurationProvider.custom_volatile_configuration.test_str())
        self.assertEqual(parser.custom_volatile_configuration.test_bool,
                         CustomConfigurationProvider.custom_volatile_configuration.test_bool())
        self.assertEqual(parser.global_configuration.t2krun,
                         CustomConfigurationProvider.global_configuration.t2krun())

    def test_create_section(self):
        CustomConfigurationProvider.create_section("test_section")
        CustomConfigurationProvider.test_section.override({'foo': 42})
        self.assertEqual(42, CustomConfigurationProvider.test_section.foo())
        CustomConfigurationProvider.create_section("test_section")  # should not raise
        CustomConfigurationProvider.delete_section("test_section")

    def test_get_sections(self):
        CustomConfigurationProvider.create_section("test_section")
        CustomConfigurationProvider.test_section.override({'foo': 42})
        self.assertIn("test_section", CustomConfigurationProvider.get_sections())
        CustomConfigurationProvider.delete_section("test_section")

    def test_dump(self):
        # Normal configuration
        print("NORMAL CONFIGURATION")
        Configuration.dump()
        print("CUSTOM CONFIGURATION")
        # Custom configuration
        CustomConfigurationProvider.dump()
        sys.stdout.flush()
        time.sleep(1)


if __name__ == '__main__':
    unittest.main()



import unittest
import numpy
import ROOT

import wagascianpy.plotting.colors
import wagascianpy.plotting.graph


class TestGraph(unittest.TestCase):

    def setUp(self):
        self.xdata_list = [0, 1, 2]
        self.ydata_list = [10, 20, 30]
        self.xdata_array = numpy.array(self.xdata_list, dtype=numpy.float64)
        self.ydata_array = numpy.array(self.ydata_list, dtype=numpy.float64)
        self.xrange = wagascianpy.plotting.graph.Range(lower_bound=0, upper_bound=2)
        self.yrange = wagascianpy.plotting.graph.Range(lower_bound=10, upper_bound=30)
        self.title = "test title"
        self.graph_id = "test_id"
        self.color = wagascianpy.plotting.colors.Colors.Azure.value

    def test_constructor(self):
        graph = wagascianpy.plotting.graph.Graph(title=self.title, graph_id=self.graph_id)
        self.assertEqual(graph.title, self.title)
        self.assertEqual(graph.id, self.graph_id)
        graph = wagascianpy.plotting.graph.Graph(title=self.title)
        self.assertEqual(graph.id, self.title)
        self.assertRaises(TypeError, wagascianpy.plotting.graph.Graph)
        self.assertIsNone(graph.xdata)
        self.assertIsNone(graph.ydata)
        self.assertEqual(wagascianpy.plotting.graph.Range(None, None), graph.xrange)
        self.assertEqual(wagascianpy.plotting.graph.Range(None, None), graph.yrange)
        self.assertEqual(wagascianpy.plotting.colors.Colors.Black.value, graph.color)
        self.assertTrue(graph.is_empty())

    def test_getter_setter(self):
        graph = wagascianpy.plotting.graph.Graph(title=self.title, graph_id=self.graph_id)
        graph.xdata = self.xdata_array
        graph.ydata = self.ydata_array
        graph.color = self.color
        graph.xrange = self.xrange
        graph.yrange = self.yrange
        self.assertListEqual(self.xdata_list, list(graph.xdata))
        self.assertListEqual(self.ydata_list, list(graph.ydata))
        graph.xdata = self.xdata_list
        graph.ydata = self.ydata_list
        self.assertListEqual(self.xdata_list, list(graph.xdata))
        self.assertListEqual(self.ydata_list, list(graph.ydata))
        self.assertEqual(self.color, graph.color)
        self.assertEqual(self.xrange, graph.xrange)
        self.assertEqual(self.yrange, graph.yrange)
        self.assertFalse(graph.is_empty())

    def test_make_tgraph(self):
        graph = wagascianpy.plotting.graph.Graph(title=self.title, graph_id=self.graph_id)
        graph.xdata = self.xdata_array
        graph.ydata = self.ydata_array
        graph.color = self.color
        graph.xrange = self.xrange
        graph.yrange = self.yrange
        tgraph = graph.make_tgraph()
        self.assertIsInstance(tgraph, ROOT.TGraph)
        self.assertEqual(len(self.xdata_array), tgraph.GetN())
        for i in range(len(self.xdata_array)):
            x = numpy.array([0], dtype=numpy.float64)
            y = numpy.array([0], dtype=numpy.float64)
            tgraph.GetPoint(i, x, y)
            self.assertEqual(self.xdata_array[i], x[0])
            self.assertEqual(self.ydata_array[i], y[0])


class TestGraph2D(unittest.TestCase):

    def setUp(self):
        self.xdata = [0, 1, 2]
        self.ydata = [[1, 1, 1], [2, 2, 2], [3, 3, 3]]
        self.title = "test title"
        self.graph_id = "test_id"
        self.graph = wagascianpy.plotting.graph.Graph2D(title=self.title, graph_id=self.graph_id)
        self.graph.xdata = self.xdata
        self.graph.ydata = self.ydata

    def test_make_tgraph(self):
        th2d = self.graph.make_tgraph()
        self.assertIsInstance(th2d, ROOT.TH2D)
        self.assertEqual(len(self.xdata), th2d.GetNbinsX())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio
#

import os
from datetime import datetime

from typing import Dict, Optional, List

import wagascianpy.database.wagascidb
import wagascianpy.utils.configuration
from wagascianpy.viewer.import_tkinter import import_filedialog, import_pygubu

pygubu = import_pygubu()


def get_time_interval_members(builder):
    # type: (pygubu.Builder) -> (datetime, datetime)
    """Build the interval period
    """
    start_hours = builder.tkvariables.__getitem__('from_hours').get()
    start_minutes = builder.tkvariables.__getitem__('from_minutes').get()
    start_seconds = builder.tkvariables.__getitem__('from_seconds').get()
    start_calendar = builder.get_object('from_calendar')
    try:
        start_datetime = start_calendar.selection
        start_time = start_datetime.replace(hour=start_hours, minute=start_minutes,
                                            second=start_seconds)
    except ValueError as error:
        raise ValueError("Error when building start time string : %s" % str(error))
    except AttributeError as error:
        raise AttributeError("Please select a month and day for start time : %s" % str(error))

    stop_hours = builder.tkvariables.__getitem__('to_hours').get()
    stop_minutes = builder.tkvariables.__getitem__('to_minutes').get()
    stop_seconds = builder.tkvariables.__getitem__('to_seconds').get()
    stop_calendar = builder.get_object('to_calendar')
    try:
        stop_datetime = stop_calendar.selection
        stop_time = stop_datetime.replace(hour=stop_hours, minute=stop_minutes,
                                          second=stop_seconds)
    except ValueError as error:
        raise ValueError("Error when building stop time string : %s" % str(error))
    except AttributeError as error:
        raise AttributeError("Please select a month and day for start time : %s" % str(error))

    if start_time > stop_time:
        raise ValueError("Stop time %s cannot be greater than start time %s"
                         % (stop_time, start_time))

    return start_time, stop_time


def check_repository_and_database_sanity(repository, database):
    # type: (str, str) -> None
    """Check WAGASCI database and repository strings sanity
    """
    if not repository:
        raise ValueError('Repository path is empty')
    if not database:
        raise ValueError('Database path is empty')
    if repository.count(':') == 1:
        tokens = repository.split(':')
        for token in tokens:
            if not token.strip(':'):
                raise ValueError('hostname or path are empty')
    elif repository.count(':') >= 2:
        raise ValueError("Too many ':' in repository string")

    if database.count(':') == 1:
        tokens = database.split(':')
        for token in tokens:
            if not token.strip(':'):
                raise ValueError('hostname or path are empty')
    elif database.count(':') >= 2:
        raise ValueError("Too many ':' in database string")

    if os.path.splitext(database)[1] != '.db':
        raise ValueError("Database extension must be .db")


def check_wagasci_libdir(wagasci_libdir):
    # type: (Optional[str]) -> Optional[str]
    if wagasci_libdir is None or wagasci_libdir == "" or not os.path.exists(wagasci_libdir):
        return None
    if wagasci_libdir.endswith('.so'):
        raise ValueError('You are supposed to provide the library directory, not the library itself')
    return wagasci_libdir


def gui_file_finder():
    # type: (...) -> str
    here = os.path.dirname(os.path.abspath(__file__))
    print(here)
    gui_file = wagascianpy.utils.configuration.conf_file_finder(filename='gui.ui', project_path=here)
    if not os.path.exists(gui_file):
        raise OSError("GUI file not found")
    print("Using GUI file %s" % gui_file)
    return gui_file


def _get_input(text):
    return input(text)


def check_input_folder(input_folder, records, batch_mode):
    # type: (Optional[str], List[Dict], bool) -> Dict[str, str]
    # 1 : Ask the user for download folder
    if not input_folder:
        if not batch_mode:
            filedialog = import_filedialog()
            if not filedialog:
                input_folder = _get_input('Download folder')
            else:
                dir_opt = {'title': 'Choose the local directory where the WAGASCI raw data has been downloaded',
                           'initialdir': os.curdir, 'mustexist': True}
                input_folder = filedialog.askdirectory(**dir_opt)
        else:
            input_folder = _get_input('Download folder')

    # 3 : check that the download folder exist
    if not os.path.exists(input_folder):
        raise OSError("WAGASCI runs download folder not found : %s" % input_folder)

    # 2 : list folders for each run
    run_dic = {}
    for record in records:
        if os.path.exists(os.path.join(input_folder, record["name"])):
            run_dic[record["name"]] = os.path.join(input_folder, record["name"])
        elif os.path.exists(os.path.join(input_folder, record["run_folder"])):
            run_dic[record["name"]] = os.path.join(input_folder, record["run_folder"].strip('/'))
        else:
            print("Run {} not found. Please download it!".format(record["name"]))

    return run_dic



RAW_TREE_NAME = "raw"
TIME_NS_TREE_NAME = "time_ns"
BSD_TREE_NAME = "bsd"
FSN_TREE_NAME = "fixed_spill_number"
PEU_TREE_NAME = "peu"
FLAGS_TREE_NAME = "detector_flags"
THRESHOLD_TREE_NAME = "hit_threshold"
TEMPHUM_TREE_NAME = "temp_hum"
HISTORY_TREE_NAME = "history"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio
#

""" Module to manage a database created with tinydb """
import operator
import os
from datetime import datetime
from random import seed, randint
import abc

from scp import SCPException
import tinydb
try:
    from tinydb.table import Document as tinydbDocument
except:
    from tinydb.database import Document as tinydbDocument
from typing import List, Optional, Union, Dict

import wagascianpy.utils.utils
import wagascianpy.database.db_record

# compatible with Python 2 *and* 3:
ABC = abc.ABCMeta('ABC', (object,), {'__slots__': ()})

_TMP_DIR = "/tmp"


###############################################################################
#                                   MyTinyDB                                  #
###############################################################################

# noinspection PyPep8,PyCallingNonCallable
class MyTinyDB(ABC):
    """Wrapper class around the tinyDB database"""

    _tmp_dir = _TMP_DIR

    def __init__(self, db_location):
        # type: (str) -> None
        # If the database resides in a remote location, copy it locally for ease
        # and speed of handling

        self._is_remote_db = False
        self._hostname = None
        self._remote_db_path = None
        self._local_db_path = None
        self._database = None
        self._is_fresh_database = False
        self._write_to_remote = False

        seed(datetime.now().timestamp())
        if ':' in db_location:
            self._is_remote_db = True
            if len(db_location.split(':', 1)) != 2:
                raise ValueError("Invalid database location : %s" % db_location)
            self._hostname = db_location.split(':', 1)[0]
            self._remote_db_path = db_location.split(':', 1)[-1]
            self._local_db_path = os.path.join(
                self._tmp_dir, str(randint(10000, 99999)) + ".db")
            try:
                print("Copying remote database %s into location %s"
                      % (self._remote_db_path, self._local_db_path))
                wagascianpy.utils.utils.scp_get(self._hostname, self._remote_db_path,
                                                self._local_db_path)
                print("Database found in remote location")
                print("Remote database copied to local location : "
                      "{}".format(self._local_db_path))
                print("Database size is %d kB" % int(os.path.getsize(self._local_db_path) / 1024))
            except SCPException as error:
                print("Creating a new database because not found in remote location : %s" % str(error))
                self._is_fresh_database = True
        else:
            self._local_db_path = db_location
            if not os.path.exists(self._local_db_path):
                print("Creating a new database because not found in local location")
                self._is_fresh_database = True
            print("using local database : %s" % self._local_db_path)

        # Open the tinydb database
        print("Opening database %s" % self._local_db_path)
        self._database = tinydb.TinyDB(self._local_db_path)

    def is_fresh_database(self):
        # type: (...) -> bool
        """ Return true if the database is empty """
        return self._is_fresh_database

    def update_database(self, run_record_list):
        # type: (List[wagascianpy.database.db_record.DBRecord]) -> None
        """ Update the tinydb database """
        for run_record in run_record_list:
            if not self.has_record(run_record.name):
                print("Inserting run %s into database" % run_record.name)
                self.insert(run_record.make_record())
        self._is_fresh_database = False
        if self._is_remote_db:
            self._write_to_remote = True

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.__del__()

    def __del__(self):
        if hasattr(self, "_database") and self._database is not None:
            self._database.close()
            self._database = None

        if hasattr(self, "_is_remote_db") and self._is_remote_db:
            if None in [self._hostname, self._remote_db_path, self._local_db_path]:
                print("Remote database location is not properly set up.")
            else:
                try:
                    if hasattr(self, "_write_to_remote") and self._write_to_remote \
                            and os.path.exists(self._local_db_path):
                        print("Writing local database %s to remote location %s"
                              % (self._local_db_path, self._remote_db_path))
                        wagascianpy.utils.utils.scp_put(self._hostname, self._local_db_path,
                                                        self._remote_db_path)
                except SCPException as error:
                    print("Could not push the database to the remote "
                          "location : {}".format(str(error)))
                    print("Temporary local database is saved here : "
                          "%s" % self._local_db_path)
                else:
                    if os.path.exists(self._local_db_path):
                        os.remove(self._local_db_path)

    def insert(self, record):
        # type: (Union[tinydbDocument, Dict]) -> None
        """Insert record into database"""
        self._database.insert(record)

    def remove(self, record):
        # type: (Union[tinydbDocument, Dict]) -> None
        """Remove a record from the database
        """
        run = tinydb.Query()
        self._database.remove(run.name == record['name'])

    def has_record(self, name):
        # type: (str) -> bool
        """Return True if the run name is in the database
        """
        run = tinydb.Query()
        return len(self._database.search(run.name == name)) > 0

    def _check_field(self, field_name):
        # type: (str) -> None
        run = tinydb.Query()
        if not self._database.search(run[field_name].exists()):
            raise ValueError("Field with name %s does not exist" % field_name)

    def _has_field(self, field_name):
        # type: (str) -> bool
        run = tinydb.Query()
        if not self._database.search(run[field_name].exists()):
            return False
        return True

    def get_duration_less_than(self, duration_h, only_good=True):
        # type: (float, bool) -> List[tinydbDocument]
        """Return all the records with duration less than duration_h"""
        duration_h = float(duration_h)
        run = tinydb.Query()
        self._check_field("duration_h")
        if only_good and self._has_field("good_run_flag"):
            return self._database.search((run.duration_h <= duration_h) &
                                         (run.good_run_flag == True))
        return self._database.search(run.duration_h <= duration_h)

    def get_duration_greater_than(self, duration_h, only_good=True):
        # type: (float, bool) -> List[tinydbDocument]
        """Return all the records with duration greater than duration_h"""
        self._check_field("duration_h")
        duration_h = float(duration_h)
        run = tinydb.Query()
        if only_good and self._has_field("good_run_flag"):
            return self._database.search((run.duration_h >= duration_h) &
                                         (run.good_run_flag == True))
        return self._database.search(run.duration_h >= duration_h)

    def get_all(self, only_good=True):
        # type: (bool) -> List[tinydbDocument]
        """Return all records
        """
        if only_good and self._has_field("good_run_flag"):
            run = tinydb.Query()
            return self._database.search(run.good_run_flag == True)
        return self._database.all()

    def get_record(self, name):
        # type: (str) -> tinydbDocument
        """Print info about run with name
        """
        run = tinydb.Query()
        return self._database.search(run.name == name)

    def get_time_interval(self, timestamp_start, timestamp_stop, only_good=True, include_overlapping=True):
        # type: (float, float, bool, bool) -> List[tinydbDocument]
        """ Get records in a certain interval of time """
        self._check_field("start_time")
        self._check_field("stop_time")
        run = tinydb.Query()
        if only_good and self._has_field("good_run_flag"):
            if timestamp_start is None or timestamp_stop is None:
                matched_records = self._database.search(run.good_run_flag == True)
            elif include_overlapping:
                matched_records = self._database.search((run.start_time <= timestamp_stop) &
                                                        (run.stop_time >= timestamp_start) &
                                                        (run.good_run_flag == True))
            else:
                matched_records = self._database.search((run.start_time >= timestamp_start) &
                                                        (run.stop_time <= timestamp_stop) &
                                                        (run.good_run_flag == True))

        else:
            if timestamp_start is None or timestamp_stop is None:
                matched_records = self._database.all()
            elif include_overlapping:
                matched_records = self._database.search((run.start_time <= timestamp_stop) &
                                                        (run.stop_time >= timestamp_start))
            else:
                matched_records = self._database.search((run.start_time >= timestamp_start) &
                                                        (run.stop_time <= timestamp_stop))
        return sorted(matched_records, key=operator.itemgetter("start_time"))

    def get_run_interval(self, run_number_start, run_number_stop=None, only_good=True):
        # type: (int, Optional[int], bool) -> List[tinydbDocument]
        """ Get records in a certain interval of run numbers """
        self._check_field("run_number")
        run = tinydb.Query()
        if only_good and self._has_field("good_run_flag"):
            if run_number_start is None:
                matched_records = self._database.search(run.good_run_flag == True)
            elif run_number_stop is None:
                matched_records = self._database.search((run.run_number == run_number_start) &
                                                        (run.good_run_flag == True))
            else:
                matched_records = self._database.search((run.run_number >= run_number_start) &
                                                        (run.run_number <= run_number_stop) &
                                                        (run.good_run_flag == True))
        else:
            if run_number_start is None:
                matched_records = self._database.all()
            elif run_number_stop is None:
                matched_records = self._database.search(run.run_number == run_number_start)
            else:
                matched_records = self._database.search((run.run_number >= run_number_start) &
                                                        (run.run_number <= run_number_stop))
        return sorted(matched_records, key=operator.itemgetter("start_time"))

    def set_tmp_dir(self, path):
        # type: (str) -> None
        """Set temporary directory path"""
        self._tmp_dir = path

    def get_last_run_number(self, only_good=True):
        # type: (bool) -> Optional[int]
        self._check_field("run_number")
        run = tinydb.Query()
        if only_good and self._has_field("good_run_flag"):
            records = self._database.search((run.run_number.exists()) & (run.good_run_flag == True))
        else:
            records = self._database.search(run.run_number.exists())
        if records is None:
            return None
        seq = [int(record["run_number"]) for record in records]
        return max(seq)

    def clear_database(self):
        # type: (...) -> None
        if hasattr(self._database, "drop_tables"):
            self._database.drop_tables()
        elif hasattr(self._database, "purge_tables"):
            self._database.purge_tables()
        else:
            raise NotImplementedError("Cannot clear the database")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio
#

# pylint: disable=broad-except

"""Unit tests for database creation"""

import os
import unittest

from wagascianpy.utils.utils import silentremove
import wagascianpy.database.wagascidb


class TestAccessWagasciDatabase(unittest.TestCase):

    def setUp(self):
        self.is_borg_repo = False
        self.update_db = False
        self.rebuild_db = True
        self.current_directory = os.path.dirname(os.path.realpath(__file__))
        silentremove(os.path.join(self.current_directory, "test.db"))

    def tearDown(self):
        silentremove(os.path.join(self.current_directory, "test.db"))

    def test_duration(self):
        """Test duration query"""
        with wagascianpy.database.wagascidb.WagasciDataBase(os.path.join(self.current_directory, "test.db"),
                                                            os.path.join(self.current_directory, "test_simple_repo"),
                                                            self.is_borg_repo, self.update_db, self.rebuild_db) as db:
            matched_runs = db.get_duration_greater_than(20)
            self.assertEqual(matched_runs[0]["name"], "physics_run_2020-01-17_15-53-04_63")

    def test_get_time_interval(self):
        """Test time interval query"""
        with wagascianpy.database.wagascidb.WagasciDataBase(os.path.join(self.current_directory, "test.db"),
                                                            os.path.join(self.current_directory, "test_simple_repo"),
                                                            self.is_borg_repo, self.update_db, self.rebuild_db) as db:
            self.assertEqual(db.get_time_interval("2020/01/16 17:00:00", "2020/01/16 23:00:00")[0]["name"],
                             "physics_run_2020-01-16_17-13-41_61")

    def test_get_run_interval(self):
        """Test run interval query"""
        with wagascianpy.database.wagascidb.WagasciDataBase(os.path.join(self.current_directory, "test.db"),
                                                            os.path.join(self.current_directory, "test_simple_repo"),
                                                            self.is_borg_repo, self.update_db, self.rebuild_db) as db:
            self.assertEqual(db.get_run_interval(61, 61)[0]["name"], "physics_run_2020-01-16_17-13-41_61")

    def test_get_all(self):
        """Test get all"""
        with wagascianpy.database.wagascidb.WagasciDataBase(os.path.join(self.current_directory, "test.db"),
                                                            os.path.join(self.current_directory, "test_simple_repo"),
                                                            self.is_borg_repo, self.update_db, self.rebuild_db) as db:
            for record in db.get_all():
                record.pretty_print()
                print("")

    def test_print_run(self):
        """Test pretty printing of run info"""
        with wagascianpy.database.wagascidb.WagasciDataBase(os.path.join(self.current_directory, "test.db"),
                                                            os.path.join(self.current_directory, "test_simple_repo"),
                                                            self.is_borg_repo, self.update_db, self.rebuild_db) as db:
            db.print_run("physics_run_2020-01-17_15-53-04_63")
            print("")

    def test_max_run_number(self):
        """Test getting the maximum run number in the database"""
        with wagascianpy.database.wagascidb.WagasciDataBase(os.path.join(self.current_directory, "test.db"),
                                                            os.path.join(self.current_directory, "test_simple_repo"),
                                                            self.is_borg_repo, self.update_db, self.rebuild_db) as db:
            self.assertEqual(db.get_last_run_number(), 65)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio

import os

import numpy as np

# ROOT
try:
    # noinspection PyPackageRequirements
    import ROOT

    ROOT.PyConfig.IgnoreCommandLineOptions = True
except ImportError:
    ROOT = None

from wagascianpy.analysis.spill import IS_GOOD_SPILL, WAGASCI_SPILL_BEAM_MODE
import wagascianpy.database.wagascidb
import wagascianpy.plotting.detector
from wagascianpy.plotting.topology import DetectorIndex
from wagascianpy.utils.utils import extract_raw_tree_name, parse_input_files
import wagascianpy.utils.treenames as treenames

DETECTOR_FLAGS_ERROR_CODE = 1001
_DETECTOR_FLAGS_BRANCH_NAME = "good_detector_flag"
_DETECTORS = [DetectorIndex.get_name_snakecase(DetectorIndex.WagasciUpstream),
              DetectorIndex.get_name_snakecase(DetectorIndex.WagasciDownstream),
              DetectorIndex.get_name_snakecase(DetectorIndex.WallMrdNorth),
              DetectorIndex.get_name_snakecase(DetectorIndex.WallMrdSouth)]


def _reset_flags(tfile):
    # type: (ROOT.TFile) -> None
    if tfile.GetListOfKeys().Contains(treenames.FLAGS_TREE_NAME):
        for key in tfile.GetListOfKeys():
            if key.GetName() == treenames.FLAGS_TREE_NAME:
                tfile.Delete("{};{}".format(treenames.FLAGS_TREE_NAME, key.GetCycle()))


def apply_detector_flags(input_path, run_number, wagasci_database):
    if ROOT is None:
        raise ImportError("No ROOT module was found!")

    if not os.path.exists(input_path):
        raise IOError("Input file %s not found" % input_path)

    run_number, detectors = parse_input_files(input_path=input_path, run_number=run_number,
                                              tree_friends=[treenames.FSN_TREE_NAME])

    with wagascianpy.database.wagascidb.WagasciDataBase(db_location=wagasci_database) as db:

        run_records = db.get_run_interval(run_number_start=run_number, only_good=False)
        if len(run_records) != 1:
            raise RuntimeError("Run {} not found in database".format(run_number))
        run_record = run_records[0]
        good_run_flag = run_record.good_run_flag

        for detector in detectors:
            enabled_difs = (dif for dif in detector if dif.is_enabled())
            for dif in enabled_difs:

                detector_name = detector.get_snake_case_name()
                detector_flag = getattr(run_record, detector_name + "_good_data_flag")

                dif.get_tree().LoadTree(0)
                dif.set_active_branches(["spill_mode", "good_spill_flag"])
                tfile = ROOT.TFile(dif.get_tree().GetCurrentFile().GetPath().split(':')[0], "UPDATE")
                _reset_flags(tfile)

                friend_tree = ROOT.TTree(treenames.FLAGS_TREE_NAME, "Detector good-bad flags")
                friend_tree.SetDirectory(tfile)
                raw_tree = getattr(tfile, extract_raw_tree_name(tfile))
                raw_tree.AddFriend(friend_tree)
                array = np.array([False], dtype=bool)
                friend_tree.Branch(_DETECTOR_FLAGS_BRANCH_NAME, array, "{}/O".format(_DETECTOR_FLAGS_BRANCH_NAME))

                for event in dif.get_tree():
                    if not good_run_flag or \
                            event.good_spill_flag != IS_GOOD_SPILL or \
                            event.spill_mode != WAGASCI_SPILL_BEAM_MODE:
                        array[0] = False
                    else:
                        array[0] = bool(detector_flag)
                    friend_tree.Fill()

                friend_tree.Write("", ROOT.TObject.kWriteDelete)
                tfile.Write("", ROOT.TObject.kWriteDelete)



import os
import time
import unittest
from random import randint

import pyfakefs.fake_filesystem_unittest
from recordclass import recordclass

import wagascianpy.analysis.analysis
import wagascianpy.utils.acq_config_xml
import wagascianpy.utils.utils


# noinspection PyPep8Naming
class TestFileSystemUtils(pyfakefs.fake_filesystem_unittest.TestCase):

    def setUp(self):
        self.topology = {"1": {"0": 32}, "3": {"0": 32}, "5": {"0": 32}, "7": {"0": 32}}
        self.setUpPyfakefs()

    def test_renametree_1(self):
        self.fs.create_file('/test_renametree/test_ecal_dif_0.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_1.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_2.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_3.raw')

        wagascianpy.utils.utils.renametree(run_root_dir='/test_renametree', run_name='test', dif_topology=self.topology)
        self.check_paths()

    def test_renametree_2(self):
        self.fs.create_file('/test_renametree/test_ecal_dif_0.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_3.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_5.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_7.raw')

        wagascianpy.utils.utils.renametree(run_root_dir='/test_renametree', run_name='test', dif_topology=self.topology)
        self.check_paths()

    def test_renametree_3(self):
        self.fs.create_file('/test_renametree/test_ecal_dif_0.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_4.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_6.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_7.raw')

        wagascianpy.utils.utils.renametree(run_root_dir='/test_renametree', run_name='test', dif_topology=self.topology)
        self.check_paths()

    def test_renametree_4(self):
        self.fs.create_file('/test_renametree/test_ecal_dif_0.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_4.raw')
        self.fs.create_file('/test_renametree/test_ecal_dif_7.raw')

        self.assertRaises(ValueError, wagascianpy.utils.utils.renametree, run_root_dir='/test_renametree',
                          run_name='test', dif_topology=self.topology)

    def check_paths(self):
        self.assertFalse(os.path.exists('/test_renametree/test_ecal_dif_0.raw'))
        self.assertTrue(os.path.exists('/test_renametree/test_ecal_dif_1.raw'))
        self.assertFalse(os.path.exists('/test_renametree/test_ecal_dif_2.raw'))
        self.assertTrue(os.path.exists('/test_renametree/test_ecal_dif_3.raw'))
        self.assertFalse(os.path.exists('/test_renametree/test_ecal_dif_4.raw'))
        self.assertTrue(os.path.exists('/test_renametree/test_ecal_dif_5.raw'))
        self.assertFalse(os.path.exists('/test_renametree/test_ecal_dif_6.raw'))
        self.assertTrue(os.path.exists('/test_renametree/test_ecal_dif_7.raw'))


def my_test_function(run_name, dif_id, max_seconds=3):
    print("Run {} DIF {} : START".format(run_name, dif_id))
    time.sleep(randint(1, max_seconds))
    print("Run {} DIF {} : RETURN".format(run_name, dif_id))
    return 0


class TestThreadUtils(unittest.TestCase):

    def setUp(self):
        self.max_threads = 4
        self.run_chains = {"test1": {}, "test2": {}, "test3": {}}
        for run_name, chain in sorted(self.run_chains.items()):
            for dif_id in range(8):
                chain[dif_id] = recordclass('Chain', ['link', 'thread'])
                chain[dif_id].thread = wagascianpy.analysis.analysis.ThreadWithReturnValue(target=my_test_function,
                                                                                           args=(run_name, dif_id,))

    def test_limit_chains(self):
        for run_name, chain in sorted(self.run_chains.items()):
            for dif_id in range(8):
                chain[dif_id].thread.start()
                wagascianpy.utils.utils.limit_chains(self.run_chains, self.max_threads)

                num_active_threads = 0
                for dif_chains in self.run_chains.values():
                    for dif_chain in dif_chains.values():
                        if dif_chain.thread.is_alive():
                            num_active_threads += 1
                print("Number of active threads = {}".format(num_active_threads))
                self.assertLessEqual(num_active_threads, self.max_threads)

    def test_join_chains(self):
        for run_name, chain in sorted(self.run_chains.items()):
            for dif_id in range(8):
                chain[dif_id].thread.start()

        wagascianpy.utils.utils.join_chains(self.run_chains)

        num_active_threads = 0
        for dif_chains in self.run_chains.values():
            for dif_chain in dif_chains.values():
                if dif_chain.thread.is_alive():
                    num_active_threads += 1
        self.assertEqual(num_active_threads, 0)


if __name__ == '__main__':
    unittest.main()



""" Set the environment variables for the WAGASCI analysis software """

# !/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2019 Pintaudi Giorgio

import shlex
import subprocess
import pprint
import os

WAGASCI_ENVIRONMENT_SCRIPT = "/opt/anpan/config/wagasci_environment.sh"


class WagasciEnvironment(dict):
    """ Class to manage WAGASCI environment variables """

    def __init__(self, path=None):
        super(WagasciEnvironment, self).__init__()
        if path is None:
            environment_script_path = WAGASCI_ENVIRONMENT_SCRIPT
        else:
            environment_script_path = path
        if os.path.exists(environment_script_path):
            command = shlex.split("env -i bash -c 'source %s && env'" % environment_script_path)
            proc = subprocess.Popen(command, stdout=subprocess.PIPE)
            for line in [line.decode('utf-8') for line in proc.stdout]:
                (key, _, value) = line.partition("=")
                try:
                    self.__setitem__(str(key), str(value).strip('\n'))
                except Exception as error:
                    print(("Failed add variable (" + str(key) + " = " + str(value) +
                           ") to the environment : " + str(error)))
            proc.communicate()
            try:
                self.__delitem__('_')
            except Exception as error:
                print("Failed to delete '_' : " + str(error))
        else:
            # print("Environment script not found : {}".format(environment_script_path))
            # print("Trying to get the environment from the shell")
            for variable_name, variable_value in os.environ.items():
                if variable_name.startswith("WAGASCI"):
                    self[str(variable_name)] = str(variable_value)

    def print_env(self):
        """ Set WAGASCI environment variables """
        pprint.pprint(self)


if __name__ == "__main__":
    ENV = WagasciEnvironment()
    for var_name, var_value in ENV.items():
        print("{} = {}".format(var_name, var_value))



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Copyright 2019 Pintaudi Giorgio, Eguchi Aoi

import json
import os
import re
import subprocess

from six import string_types
from typing import Tuple

import wagascianpy.utils
from wagascianpy.pyrmod.pyrmod import PyrameSlowModule
from wagascianpy.utils.environment import WagasciEnvironment


###############################################################################
#                              WagasciConfig                                  #
###############################################################################

class WagasciConfig(PyrameSlowModule):
    _wg_config_module_name = "wg_config"

    def __init__(self, *arg, **kwargs):
        super(WagasciConfig, self).__init__(module_name=self._wg_config_module_name, *arg, **kwargs)


###############################################################################
#                          create_acq_config_xml                              #
###############################################################################

def create_acq_config_xml(topology, acq_config_path=None, acq_config_xml=None, running_dir=None, run_top_dir=None,
                          dif_mapping_file=None, mac_mapping_file=None, simulate=False, use_rundb=False):
    # type: (...) -> Tuple[int, str]
    """ Create Pyrame configuration XML file for the data taking acquisition
            :param topology: topology string
            :param acq_config_path: directory where to create the Pyrame XML configuration file
            :param acq_config_xml: file name of the Pyrame XML configuration file
            :param running_dir: directory were the data is temporarily stored during data acquisition
            :param run_top_dir: directory were the data is copied after data acquisition
            :param dif_mapping_file: JSON file containing the mapping between DIF ID number and GDCC port number
            :param mac_mapping_file: JSON file containing the mapping between GDCC ID number and GDCC MAC address
            :param simulate: in the configuration file set the simulation fields to true.
             Different from the Pyrame module call simulation.
            :param use_rundb: If set the Pyrame rundb module is activated

    """

    _conf_string = r'wg_config('
    for var_name, var_value in locals().items():
        if var_value not in [None, ""] and not var_name.startswith('_'):
            if isinstance(var_value, string_types):
                var_value = var_value.replace(',', r'\,')
            _conf_string += r'{}={},'.format(var_name, var_value)
    _conf_string.rstrip(',')
    _conf_string += r')'

    try:
        WagasciConfig(conf_string=_conf_string)
    except RuntimeError as err:
        return 0, str(err)

    return 1, "ok"


###############################################################################
#                                 get_net_if                                  #
###############################################################################

def get_net_if():
    # type: (...) -> str
    """Return the name of the network interface connected to the 192.168.10.0/24 LAN"""
    gdcc_pc_dev = None
    get_dev_command = "ip -o addr show"
    process = subprocess.Popen(get_dev_command.split(), stdout=subprocess.PIPE)
    ps_output, error = process.communicate()
    if error:
        raise ValueError("Error when looking for network interface : %s" % error)
    for line in ps_output.strip().decode().splitlines():
        if re.search("192.168.10", line):
            fields = line.strip().split()
            gdcc_pc_dev = fields[1]
            break
    if gdcc_pc_dev is None:
        print("could not find a network interface connected "
              "to the 192.168.10.0 LAN")
        gdcc_pc_dev = "eth0"
    return gdcc_pc_dev


###############################################################################
#                          acqconfigxml_file_finder                           #
###############################################################################

def acqconfigxml_file_finder(run_root_dir, run_name):
    # type: (str, str) -> str
    env = wagascianpy.utils.environment.WagasciEnvironment()
    path1 = os.path.join(run_root_dir, run_name + ".xml")
    path2 = os.path.join(run_root_dir, os.path.basename(env['WAGASCI_ACQCONFIGDIR']), env['WAGASCI_ACQCONFIGXML'])
    path3 = os.path.join(run_root_dir, env['WAGASCI_ACQCONFIGXML'])
    if os.path.exists(path1):
        acqconfigxml = path1
    elif os.path.exists(path2):
        acqconfigxml = path2
    elif os.path.exists(path3):
        acqconfigxml = path3
    else:
        raise EnvironmentError("XML acquisition configuration file not found. "
                               "Run root dir : {}, Run name : {}".format(run_root_dir, run_name))
    return acqconfigxml



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2019 Pintaudi Giorgio, Eguchi Aoi

""" Script to analyze the S-curve data of the SPIROC2D ASIC """

# Python modules
import json
import argparse
import multiprocessing
import os

import ROOT
from bitarray import bitarray

# WAGASCI modules
from wagascianpy.analysis.analysis import WagasciAnalysis
from wagascianpy.utils.environment import WagasciEnvironment
from wagascianpy.utils.utils import list_dir_with_integer

MAX_NB_THREAD_CHAINS = 16


# ================================================================ #
#                                                                  #
#                        Analysis functions                        #
#                                                                  #
# ================================================================ #

def scurve_decoder(wagasci_lib, threshold_dir, acq_name, dif, n_chips, overwrite_flag=True):
    """ wgDecoder for the scurve_run.py script """
    wg_ana = WagasciAnalysis(wagasci_lib)
    dif = int(dif)
    n_chips = int(n_chips)
    input_raw_file = os.path.join(threshold_dir, "RawData", "%s_ecal_dif_%d.raw" % (acq_name, dif))
    output_dir = os.path.join(threshold_dir, "wgDecoder")
    compatibility_mode = False
    return wg_ana.decoder(input_raw_file, "", output_dir, overwrite_flag, compatibility_mode, dif, n_chips)


def scurve_make_hist(wg_ana, threshold_dir, acq_name, dif, acq_config_xml):
    """ wgMakeHist for the pedestal_run.py script """
    dif = int(dif)
    input_tree_file = os.path.join(threshold_dir, "wgDecoder", "%s_ecal_dif_%d_tree.root" % (acq_name, dif))
    output_dir = os.path.join(threshold_dir, "wgMakeHist")
    flags = bitarray('0' * 9, endian='big')
    flags[7] = True  # dark noise
    flags[0] = True  # overwrite
    ul_flags = int(flags.to01(), 2)
    return wg_ana.make_hist(input_tree_file, acq_config_xml, output_dir, ul_flags, dif)


def scurve_ana_hist(wg_ana, threshold_dir, acq_name, dif, acq_config_xml):
    """ wgAnaHist for the pedestal_run.py script """
    dif = int(dif)
    input_hist_file = os.path.join(threshold_dir, "wgMakeHist", "%s_ecal_dif_%d_hist.root" % (acq_name, dif))
    output_dir = os.path.join(threshold_dir, "wgAnaHist", "Xml", "dif%d" % dif)
    output_img_dir = os.path.join(threshold_dir, "wgAnaHist", "Images", "dif%d" % dif)
    flags = bitarray('0' * 8, endian='big')
    flags[7] = True  # overwrite
    flags[6] = bool(acq_config_xml)
    flags[5] = False  # print
    flags[4] = True  # Dark noise
    ul_flags = int(flags.to01(), 2)
    return wg_ana.ana_hist(input_hist_file, acq_config_xml, output_dir, output_img_dir, ul_flags, dif)


def scurve_ana_hist_summary(wg_ana, threshold_dir, dif):
    """ wgAnaHistSummary for the pedestal_run.py script """
    dif = int(dif)
    input_dir = os.path.join(threshold_dir, "wgAnaHist", "Xml", "dif%d" % dif)
    output_xml_dir = os.path.join(threshold_dir, "wgAnaHistSummary", "Xml", "dif%d" % dif)
    output_img_dir = os.path.join(threshold_dir, "wgAnaHistSummary", "Images", "dif%d" % dif)
    flags = bitarray('0' * 8, endian='big')
    flags[7] = True  # overwrite
    flags[5] = False  # print
    flags[4] = True  # Dark noise
    ul_flags = int(flags.to01(), 2)
    return wg_ana.ana_hist_summary(input_dir, output_xml_dir, output_img_dir, ul_flags)


def scurve_scurve(wg_ana, run_root_dir):
    """ wgScurve for the scurve_run.py script """
    output_dir = os.path.join(run_root_dir, "wgScurve", "Xml")
    output_img_dir = os.path.join(run_root_dir, "wgScurve", "Images")
    paranoid_mode = False
    return wg_ana.scurve(run_root_dir, output_dir, output_img_dir, paranoid_mode)


# ================================================================ #
#                                                                  #
#                          scurve_analysis                         #
#                                                                  #
# ================================================================ #

def scurve_analysis(run, decoder=True, makehist=True, anahist=True, overwrite=False):
    """ S-curve run """

    decoder = bool(decoder)
    makehist = bool(makehist)
    anahist = bool(anahist)
    overwrite = bool(overwrite)

    # Environmental variables
    env = WagasciEnvironment()
    wagasci_lib = env["WAGASCI_LIBDIR"]

    if not os.path.exists(run):
        run = os.path.join(env['WAGASCI_CALIBDATADIR'], run)

    run_name = os.path.basename(run)
    run_root_dir = run
    acq_config_path = os.path.join(run_root_dir, os.path.basename(env['WAGASCI_ACQCONFIGDIR']))
    acq_config_xml = os.path.join(acq_config_path, env['WAGASCI_ACQCONFIGXML'])
    if not os.path.exists(acq_config_xml):
        print("Acquisition configuration XML file not found : %s" % acq_config_xml)

    # =========================================================== #
    #                        ANALYZE DATA                         #
    # =========================================================== #

    wg_ana = WagasciAnalysis(wagasci_lib)
    topology_string, pointer = wg_ana.get_dif_topology(acq_config_xml)
    dif_topology = json.loads(topology_string)
    wg_ana.free_topology(pointer)
    wg_ana.enable_thread_safety()
    del wg_ana

    ###########################################################################
    #                                 decoder                                 #
    ###########################################################################

    if decoder:

        pool = multiprocessing.Pool(processes=multiprocessing.cpu_count())
        results = {}

        idac_dirs = list_dir_with_integer(run_root_dir)

        for idac, idac_dir in idac_dirs:
            threshold_dirs = list_dir_with_integer(os.path.join(run_root_dir, idac_dir))
            for threshold, threshold_dir in threshold_dirs:
                threshold_dir = os.path.join(run_root_dir, idac_dir, threshold_dir)
                acq_name = "%s_iDAC%s_threshold%s" % (run_name, idac, threshold)
                for dif in dif_topology:
                    filepath = os.path.join(threshold_dir, "wgDecoder", "%s_ecal_dif_%s_tree.root" % (acq_name, dif))
                    if not overwrite and os.path.exists(filepath):
                        tfile = ROOT.TFile(filepath, "READ")
                        if not tfile.IsZombie():
                            print("Skipping %s" % filepath)
                            continue
                        else:
                            print("Removing zombie file %s" % filepath)
                            os.remove(filepath)
                    n_chips = len(dif_topology[dif])
                    result = pool.apply_async(scurve_decoder,
                                              args=(wagasci_lib, threshold_dir, acq_name, dif, n_chips, overwrite))
                    results[filepath] = result
                # dif loop
            # threshold loop
        # idac loop

        # Wait until all the threads have returned
        have_errors = False
        for filepath, result in results.items():
            result = result.get()
            if result != 0:
                have_errors = True
                print("File {} returned error {}".format(filepath, result))

        if have_errors:
            choice = input("\nErrors happened during decoding : whould you like to continue? (yes, no) : ")
            if choice.lower() in ["n", "no"]:
                return -1

        pool.close()
        pool.terminate()
        pool.join()

    ###########################################################################
    #                                make_hist                                #
    ###########################################################################

    if makehist:

        idac_dirs = list_dir_with_integer(run_root_dir)

        for idac, idac_dir in idac_dirs:
            threshold_dirs = list_dir_with_integer(os.path.join(run_root_dir, idac_dir))
            for threshold, threshold_dir in threshold_dirs:
                threshold_dir = os.path.join(run_root_dir, idac_dir, threshold_dir)
                acq_name = "%s_iDAC%s_threshold%s" % (run_name, idac, threshold)
                actual_acq_config_xml = os.path.join(threshold_dir, "RawData", "%s.xml" % acq_name)
                for dif in dif_topology:
                    filepath = os.path.join(threshold_dir, "wgMakeHist", "%s_ecal_dif_%s_hist.root" % (acq_name, dif))
                    if not overwrite and os.path.exists(filepath):
                        tfile = ROOT.TFile(filepath, "READ")
                        if not tfile.IsZombie():
                            print("Skipping %s" % filepath)
                            continue
                        else:
                            print("Removing zombie file %s" % filepath)
                            os.remove(filepath)
                    wg_ana = WagasciAnalysis(wagasci_lib)
                    result = scurve_make_hist(wg_ana=wg_ana, threshold_dir=threshold_dir, acq_name=acq_name,
                                              dif=dif, acq_config_xml=actual_acq_config_xml)
                    if result != 0:
                        print("wgMakeHist returned error code %d" % result)
                        exit(result)
                    del wg_ana
                # dif loop
            # threshold loop
        # idac loop

    #############################################################################
    #                                   ana_hist                                #
    #############################################################################

    if anahist:

        idac_dirs = list_dir_with_integer(run_root_dir)

        for idac, idac_dir in idac_dirs:
            threshold_dirs = list_dir_with_integer(os.path.join(run_root_dir, idac_dir))
            for threshold, threshold_dir in threshold_dirs:
                threshold_dir = os.path.join(run_root_dir, idac_dir, threshold_dir)
                acq_name = "%s_iDAC%s_threshold%s" % (run_name, idac, threshold)
                actual_acq_config_xml = os.path.join(threshold_dir, "RawData", "%s.xml" % acq_name)
                for dif in dif_topology:
                    anahist_dirname = os.path.join(threshold_dir, "wgAnaHist", "Xml", "dif%s" % dif)
                    if not overwrite and os.path.exists(anahist_dirname):
                        print("Skipping %s" % anahist_dirname)
                        continue
                    wg_ana = WagasciAnalysis(wagasci_lib)
                    result = scurve_ana_hist(wg_ana=wg_ana, threshold_dir=threshold_dir, acq_name=acq_name,
                                             dif=dif, acq_config_xml=actual_acq_config_xml)
                    if result != 0:
                        print("wgAnaHist returned error code %d" % result)
                        exit(result)
                    del wg_ana
                # dif loop
            # peu loop
        # input_dac loop

    #############################################################################
    #                               ana_hist_summary                            #
    #############################################################################

    if anahist:

        idac_dirs = list_dir_with_integer(run_root_dir)

        for idac, idac_dir in idac_dirs:
            threshold_dirs = list_dir_with_integer(os.path.join(run_root_dir, idac_dir))
            for threshold, threshold_dir in threshold_dirs:
                threshold_dir = os.path.join(run_root_dir, idac_dir, threshold_dir)
                for dif in dif_topology:
                    anahist_dirname = os.path.join(threshold_dir, "wgAnaHistSummary", "Xml", "dif%s" % dif)
                    if not overwrite and os.path.exists(anahist_dirname):
                        print("Skipping %s" % anahist_dirname)
                        continue
                    wg_ana = WagasciAnalysis(wagasci_lib)
                    result = scurve_ana_hist_summary(wg_ana=wg_ana, threshold_dir=threshold_dir, dif=dif)
                    if result != 0:
                        print("wgAnaHistSummary returned error code %d" % result)
                        exit(result)
                    del wg_ana
                # dif loop
            # peu loop
        # input_dac loop

    #######################################################################
    #                           scurve analysis                           #
    #######################################################################

    wg_ana = WagasciAnalysis(wagasci_lib)
    result = scurve_scurve(wg_ana=wg_ana, run_root_dir=run_root_dir)
    del wg_ana
    if result != 0:
        print("wgScurve returned error code %d" % result)


if __name__ == "__main__":
    PARSER = argparse.ArgumentParser(usage='use "python %(prog)s --help" for more information',
                                     argument_default=None, description='Analyze the Scurve calibration data')

    PARSER.add_argument('-f', '--run', metavar='<run name or path>', type=str, required=True,
                        help='Name or path of the run to analyze. If only the run name is given the run must be inside '
                             'the ${WAGASCI_CALIBDATADIR} folder.')
    PARSER.add_argument('-d', '--disable-decoder', dest='decoder', action='store_false',
                        required=False, default=True, help="Disable the wgDecoder.")
    PARSER.add_argument('-m', '--disable-makehist', dest='makehist', action='store_false',
                        required=False, default=True, help="Disable the wgMakeHist.")
    PARSER.add_argument('-a', '--disable-anahist', dest='anahist', action='store_false',
                        required=False, default=True, help="Disable the wgAnaHist and wgAnaHistSummary.")
    PARSER.add_argument('-r', '--overwrite', dest='overwrite', action='store_true',
                        required=False, default=False, help="Overwrite flag.")

    ARGS = PARSER.parse_args()

    RUN = ARGS.run
    DECODER = ARGS.decoder
    MAKEHIST = ARGS.makehist
    ANAHIST = ARGS.anahist
    OVERWRITE = ARGS.overwrite

    scurve_analysis(RUN, DECODER, MAKEHIST, ANAHIST, OVERWRITE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio

# Python modules
import argparse
import textwrap

# user
import wagascianpy.analysis.apply_detector_flags

###############################################################################
#                                  arguments                                  #
###############################################################################

if __name__ == "__main__":
    PARSER = argparse.ArgumentParser(usage='use "python %(prog)s --help" for more information',
                                     argument_default=None, description=textwrap.dedent('''\
                                     Apply the detector flags as an additional TTree called detector_flags'''))

    PARSER.add_argument('-f', '--input-file', metavar='<WAGASCI decoded file>', dest='input_file',
                        type=str, required=True, help=textwrap.dedent('''\
                        Path to a WAGASCI decoded file *_ecal_dif_X_tree.root or a directory containing those files.
                        If it is a directory all the files ending in *_ecal_dif_X_tree.root inside it will be analyzed.
                        If the "r" option is set the directory is traversed recursively.  
                        '''))

    PARSER.add_argument('-n', '--run-number', metavar='<WAGASCI run number>', dest='run_number',
                        type=int, required=False, default=-1, help=textwrap.dedent('''\
                        Run number. If not given the run number is guessed from the file or folder name.
                        '''))

    PARSER.add_argument('-b', '--wagasci-database', metavar='<WAGASCI database>', dest='wagasci_database', type=str,
                        required=True, help=textwrap.dedent('''\
                        Path to the WAGASCI run database file. This file is created by the 
                        Wagasci Database Viewer (wagasciviewer.py) program and is usually called wagascidb.db.
                        '''))

    ARGS = PARSER.parse_args()

    INPUT_FILE = ARGS.input_file
    RUN_NUMBER = ARGS.run_number
    WAGASCI_DATABASE = ARGS.wagasci_database

    wagascianpy.analysis.apply_detector_flags.apply_detector_flags(INPUT_FILE, RUN_NUMBER, WAGASCI_DATABASE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio

import os
import re

from six import string_types
from typing import Optional, Dict, Tuple, List

# Tries to guess the number of cores of the machine to correctly extimate the maximum number of threads to spawn
try:
    from multiprocessing import cpu_count
except ImportError as err:
    if "multiprocessing" in repr(err):
        # noinspection PyUnresolvedReferences
        from os import cpu_count
    else:
        raise

import wagascianpy.analysis.analysis as analysis
import wagascianpy.analysis.analyzer as analyzer
import wagascianpy.utils.environment as environment
import wagascianpy.utils.utils as utils

MAX_THREADS = 8


class Program(object):
    """
    Class to analyze a list of runs as an uninterrupted job. The usage scenario is when
    you need to analyze multiple runs in one batch and do not want to monitor the chain
    continuously.
    """

    def __init__(self, wagasci_libdir=None):
        # type: (Optional[str]) -> None
        """
        :param wagasci_libdir: path to the directory containing the WAGASCI library
        """

        global MAX_THREADS
        self._stop_on_exception = True
        self._enforce_dependencies = True
        self._run_dict = {}
        self._output_dir_same_as_input = True
        self._save_dict = {}
        self._multiple_run_analyzer_save_location = None
        self._analyzer_factories = []
        self._wagasci_libdir = wagasci_libdir

        # Enable thread safety
        env = environment.WagasciEnvironment()
        if self._wagasci_libdir is None:
            try:
                self._wagasci_libdir = env['WAGASCI_LIBDIR']
            except KeyError as error:
                raise KeyError("Could not find WAGASCI library directory : %s" % error)
        wagascilib_call = analysis.WagasciAnalysis(self._wagasci_libdir)
        wagascilib_call.enable_thread_safety()
        nproc = cpu_count()
        if not nproc:
            MAX_THREADS = 1
        elif 1 <= nproc <= MAX_THREADS:
            MAX_THREADS = nproc

    def _set_input_output(self, is_first_analyzer, run_name, run_root_dir):
        # type: (bool, str, str) -> Tuple[str, str]
        """setup the input directory and output directory for each analyzer"""
        if is_first_analyzer:
            if self._output_dir_same_as_input:
                output_dir = run_root_dir
            else:
                output_dir = self._save_dict[run_name]
        else:
            if self._output_dir_same_as_input:
                output_dir = run_root_dir
            else:
                output_dir = self._save_dict[run_name]
                run_root_dir = self._save_dict[run_name]
        return output_dir, run_root_dir

    # noinspection PyMethodMayBeStatic
    def _get_run_number(self, run_name):
        # type: (str) -> int
        """Parse run name for the run number"""
        match = re.search(r'.*_([\d]+)$', run_name)
        run_number = None if match is None else int(match.group(1))
        return run_number

    def _get_run_numbers(self):
        # type: (...) -> List[int]
        """Parse run names for the run numbers"""
        run_numbers = []
        for run_name in self._run_dict:
            run_numbers.append(self._get_run_number(run_name))
        return run_numbers

    def start(self):
        # type: (...) -> None
        """
        Start the program execution. For each analyzer that is to be applied to the input runs, there are two
        possibilities. One is that the analyzer accepts as input a single run. In such a case, the runs are looped
        over and fed one by one to the analyzer. The other one is that the analyzer accepts as input multiple runs.
        In such a case, all the runs in the selected interval are directly passed to the analyzer.

        It is assumed that for all the analyzers (but the raw data decoder) the input files are the same as the
        output files. In other words, the output information is appended to the ROOT input files or included in the
        same directory.

        Depending on the analyzer, multiple threads might be spawn, one for each input run. By design not all parts
        of ROOT are thread-safe, so it is not possible to run some analyzers in a multithreaded environment.

        :return: None
        """

        # setup first analyzer flags
        is_first_analyzer = {}
        for run_name in self._run_dict.keys():
            is_first_analyzer[run_name] = True
        # setup WagasciAnalysis thread chains
        run_chains = {}
        for run_name in self._run_dict.keys():
            run_chains[run_name] = {}
        # loop over the analyzers
        for analyzer_factory in self._analyzer_factories:
            print('Found analyzer {}'.format(analyzer_factory.name))
            if analyzer_factory.input_type == analyzer.AnalyzerInputType.single_run:
                # loop over the single runs
                for run_name, run_root_dir in sorted(self._run_dict.items()):
                    try:
                        # setup the input directory and output directory for each analyzer
                        output_dir, run_root_dir = self._set_input_output(is_first_analyzer=is_first_analyzer[run_name],
                                                                          run_name=run_name, run_root_dir=run_root_dir)
                        is_first_analyzer[run_name] = False
                        # get the run number from the run name
                        run_number = self._get_run_number(run_name)
                        # Get the analyzer from the factory
                        run_analyzer = analyzer_factory.get_analyzer(run_root_dir=run_root_dir, run_name=run_name,
                                                                     run_number=run_number, output_dir=output_dir)
                        # Spawn run analyzer
                        print('Applying "{}" analyzer on run "{}"'.format(run_analyzer.name, run_name))
                        run_analyzer.spawn(run_chains[run_name])
                        print("Total number of thread chains : {}".format(utils.count_threads(run_chains)))
                        # Limit number of threads
                        if analyzer_factory.threading_type == analyzer.AnalyzerThreadingType.multi_threaded:
                            utils.limit_chains(run_chains, MAX_THREADS)
                        else:
                            utils.join_single_chain(run_chains[run_name])
                    except Exception as exception:
                        if self._stop_on_exception:
                            raise exception
                        else:
                            print("Run {} failed with exception : {}".format(run_name, str(exception)))
            elif analyzer_factory.input_type == analyzer.AnalyzerInputType.multiple_runs:
                chain = {}
                # set the name for the multirun analyzer
                run_name = analyzer_factory.name
                run_numbers = self._get_run_numbers()
                min_run_number = min(run_numbers)
                max_run_number = max(run_numbers)
                if min_run_number is not None and max_run_number is not None:
                    run_name = "{}_from_{}_to_{}".format(run_name, min_run_number, max_run_number)
                try:
                    # set input and output directories
                    if all(is_first_analyzer.values()) or not self._save_dict:
                        run_root_dir = sorted(list(self._run_dict.values()))
                    else:
                        run_root_dir = sorted(list(self._save_dict.values()))
                    if self._multiple_run_analyzer_save_location is None:
                        raise ValueError("To use a multiple run analyzer you must set its save location first")
                    output_dir = self._multiple_run_analyzer_save_location
                    # Get analyzer from factory
                    ana = analyzer_factory.get_analyzer(run_root_dir=run_root_dir, run_name=run_name,
                                                        run_number=None, output_dir=output_dir)
                    print("Applying %s analyzer on runs from %s to %s"
                          % (ana.name, min_run_number, max_run_number))
                    # Spawn all the threads and join them rightaway
                    ana.spawn(chain)
                    utils.join_single_chain(chain)
                except Exception as exception:
                    if self._stop_on_exception:
                        raise exception
                    else:
                        print("{} failed with exception : {}".format(run_name, str(exception)))
            else:
                raise NotImplementedError("Analyzer type not recognized %s" % analyzer_factory.input_type)
            utils.join_chains(run_chains)
            for run_name in self._run_dict.keys():
                run_chains[run_name] = {}

    @property
    def multiple_runs_analyzer_save_location(self):
        # type: (...) -> str
        """
        In case of analyzers which need to analyze multiple runs at once and store the results in another directory,
        the user needs to specify it. One example is the wgAdcCalib program that needs to store the gain and dark
        noise history TTrees in some external directory.
        :return: output directory for analyzers that process multiple runs
        """
        return self._multiple_run_analyzer_save_location

    @multiple_runs_analyzer_save_location.setter
    def multiple_runs_analyzer_save_location(self, location):
        # type: (str) -> None
        """
        In case of analyzers which need to analyze multiple runs at once and store the results in another directory,
        the user needs to specify it. One example is the wgAdcCalib program that needs to store the gain and dark
        noise history TTrees in some external directory.
        :param location: output directory for analyzers that process multiple runs
        :return: None
        """
        if not isinstance(location, string_types):
            raise TypeError("multiple run analyzersave location must be a string path")
        if not os.path.exists(location):
            utils.mkdir_p(location)
        self._multiple_run_analyzer_save_location = location

    def set_run_location(self, run_dict):
        # type: (Dict[str, str]) -> None
        """
        Set the input runs dictionary
        :param run_dict: input runs dictionary (key: run name, value: run path)
        :return: None
        """
        self._run_dict = run_dict

    def get_run_location(self):
        # type: (...) -> Dict[str, str]
        """
        Get the input runs dictionary
        :return: input runs dictionary (key: run name, value: run path)
        """
        return self._run_dict

    def get_save_location(self):
        # type: (...) -> Dict[str, str]
        """
        Get a custom location where to store each run decoded data.
        :return: Dictionary where the key is the run name and the value is the path of the folder where the decoded
                 data is to be stored

        """
        return self._save_dict

    def set_save_location(self, save_dict):
        # type: (Dict[str, str]) -> None
        """
        Set a custom location where to store each run decoded data.
        :param save_dict: Dictionary where the key is the run name and the value is the path of the folder where the
                          decoded data is to be stored
        :rtype: None
        """
        self._save_dict = save_dict
        for run_name in self._run_dict:
            if run_name not in save_dict:
                raise KeyError("The save location dictionary does not contain the run named '%s'" % run_name)
        self._output_dir_same_as_input = False

    def output_dir_same_as_input(self):
        # type: (...) -> None
        """
        Set the output directory as the same as the input directory
        :return: None
        """
        self._save_dict.clear()
        self._output_dir_same_as_input = True

    def _check_dependencies(self, factory):
        # type: (analyzer.AnalyzerFactory) -> analyzer.AnalyzerFactory
        """
        Check that the order in which the analyzers are called is correct (that each analyzer dependency is satisfied)
        :param factory: factory
        :return: same factory
        """
        if factory.depends and factory.depends not in [f.name for f in self._analyzer_factories]:
            raise RuntimeError("{} depends on {} but not found".format(factory.name, factory.depends))
        return factory

    def add_step(self, name, **kwargs):
        # type: (str, ...) -> None
        """
        Add an analyzer to the list of analyzers of the program
        :param name: analyzer name
        :param kwargs: arguments to pass to the analyzer
        :return: None
        """
        producer = analyzer.AnalyzerFactoryProducer(self._wagasci_libdir)
        factory = producer.get_factory(name, **kwargs)
        if self._enforce_dependencies:
            analyzer_factory = self._check_dependencies(factory)
        else:
            analyzer_factory = factory
        self._analyzer_factories.append(analyzer_factory)

    def do_not_stop_on_exception(self):
        # type: (...) -> None
        """
        Do not stop the program execution if an analyzer fails
        :return: None
        """
        self._stop_on_exception = False

    def stop_on_exception(self):
        # type: (...) -> None
        """
        Stop the program execution if an analyzer fails
        :return: None
        """
        self._stop_on_exception = True

    def enforce_dependencies(self):
        # type: (...) -> None
        """
        Make sure that each analyzer is called in the right order
        :return: None
        """
        self._enforce_dependencies = True

    def do_not_enforce_dependencies(self):
        # type: (...) -> None
        """
        Allow an analyzer to be called even if the previous one is not present
        :return: None
        """
        self._enforce_dependencies = False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio

import argparse
import textwrap
from typing import List, Optional

import inflection

import wagascianpy.plotting.topology as topol


def parse_args(args):
    # type: (List[str]) -> argparse.Namespace
    """
    Parse arguments from command line or configuration file
    :param args: raw arguments
    :return: parsed arguments
    """
    parser = argparse.ArgumentParser(usage='use "python %(prog)s --help" for more information',
                                     argument_default=None, description=textwrap.dedent('''\
                                     Produce spill history plots. If the stop time or stop run is not provided
                                      the present time or the last run are assumed. If both time and run number
                                      are provided the run number is preferred.'''))

    parser.add_argument('--output-string', metavar='<output string>', dest='output_string',
                        type=str, required=True, help=textwrap.dedent('''\
                        Output string to prepend to the file names
                        '''))

    parser.add_argument('--output-path', metavar='<output path>', dest='output_path',
                        type=str, required=True, help=textwrap.dedent('''\
                        Directory where to save the output plots
                        '''))
    parser.add_argument('--delivered-pot', dest='delivered_pot', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the POT delivered by the beam line in the selected interval of time (or runs)'''))

    parser.add_argument('--accumulated-pot', dest='accumulated_pot', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the POT accumulated by the WAGASCI detectors in the selected interval of time (or 
                        runs)'''))

    parser.add_argument('--bsd-spill-history', dest='bsd_spill_history', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the BSD spill number in the selected interval of time (or runs)'''))

    parser.add_argument('--wagasci-spill-history', dest='wagasci_spill_history', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the WAGASCI spill number history in time for the selected interval of time (or 
                        runs)'''))

    parser.add_argument('--wagasci-spill-number', dest='wagasci_spill_number', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the WAGASCI spill number for the selected interval of time (or runs)'''))

    parser.add_argument('--wagasci-fixed-spill', dest='wagasci_fixed_spill', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the WAGASCI fixed spill number in the selected interval of time (or runs)'''))

    parser.add_argument('--temperature', dest='temperature', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the WAGASCI and WallMRD temperature in the selected interval of time (or runs)'''))

    parser.add_argument('--humidity', dest='humidity', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the WAGASCI and WallMRD humidity in the selected interval of time (or runs)'''))

    parser.add_argument('--gain-history', dest='gain_history', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the MPPC gain history in the selected interval of time (or runs)'''))

    parser.add_argument('--dark-noise-history', dest='dark_noise_history', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the MPPC dark noise history in the selected interval of time (or runs)'''))

    parser.add_argument('--threshold-history', dest='threshold_history', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the MPPC hit threshold history in the selected interval of time (or runs)'''))

    parser.add_argument('--bcid-history', dest='bcid_history', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot the BCID history in the selected interval of time (or runs)'''))

    parser.add_argument('--all', dest='all', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        If set plot all available plots are produced for the selected interval of time (or runs)'''))

    parser.add_argument('--run-markers', dest='run_markers', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''If set the WAGASCI runs are marked'''))

    parser.add_argument('--maintenance-markers', dest='maintenance_markers', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''If set the maintenance days are marked'''))

    parser.add_argument('--trouble-markers', dest='trouble_markers', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''If set the electronics and DAQ trouble events are 
                        marked'''))

    parser.add_argument('--bsd-markers', dest='bsd_markers', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''If set the BSD files are marked'''))

    parser.add_argument('--save-tfile', dest='save_tfile', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''If set save the TCanvas inside a ROOT file'''))

    parser.add_argument('--wagasci-database', metavar='<WAGASCI run database>', dest='wagasci_database',
                        type=str, required=False, help=textwrap.dedent('''\
                        Path to the WAGASCI run database file. This file is created by the 
                        Wagasci Database Viewer (wagasciviewer.py) program and is usually called wagascidb.db.
                        (default: %(default)s)
                        '''))

    parser.add_argument('--wagasci-decoded-location', metavar='<WAGASCI runs directory>',
                        dest='wagasci_decoded_location', type=str, required=False, help=textwrap.dedent('''\
                        Path to the directory containing all the WAGASCI decoded runs. (default: %(default)s)
                        '''))

    parser.add_argument('--bsd-database', metavar='<BSD database>', dest='bsd_database',
                        type=str, required=False, help=textwrap.dedent('''\
                        Path to the BSD database file. This file is created by the Wagasci Database Viewer
                         (wagasciviewer.py) program and is usually called bsddb.db. (default: %(default)s)
                        '''))

    parser.add_argument('--bsd-download-location', metavar='<BSD files directory>',
                        dest='bsd_download_location', type=str, required=False, help=textwrap.dedent('''\
                        Path to the directory containing all the BSD root files.
                        '''))

    parser.add_argument('--bsd-repository', metavar='<BSD repository location>', dest='bsd_repository', type=str,
                        nargs=1, required=False, help=textwrap.dedent('''
                        Same as --bsd-download-location'''))

    parser.add_argument('--t2krun', metavar='<T2K run>', dest='t2krun', type=int,
                        help='T2K run number (default is 10)', default=10, required=False)

    parser.add_argument('--history-location', metavar='<History location',
                        dest='history_location', type=str, required=False, help=textwrap.dedent('''
                        Local folder where to store output of wgAdcCalib program.(default: %(default)s)'''))

    parser.add_argument('--start-time', metavar='<start time>', dest='start_time', type=str,
                        help='Start date and time in the format "%%Y/%%m/%%d %%H:%%M:%%S"', default=None,
                        required=False)

    parser.add_argument('--stop-time', metavar='<stop time>', dest='stop_time', type=str,
                        help='Stop date and time in the format "%%Y/%%m/%%d %%H:%%M:%%S"', default=None,
                        required=False)

    parser.add_argument('--start-run', metavar='<start run>', dest='start_run', type=int,
                        help='start run number', default=None, required=False)

    parser.add_argument('--stop-run', metavar='<stop run>', dest='stop_run', type=int,
                        help='Stop run number', default=None, required=False)

    parser.add_argument('--topology', metavar='<topology>', dest='topology', type=str, required=False,
                        default=None, help=textwrap.dedent('''
                        Select the subdetectors. List their names separated by comma and do NOT mix the two groups:
                         GROUP BY SUBDETECTOR = (wagasci_upstream, wagasci_downstream, wallmrd_north, wallmrd_south) or
                         GROUP BY DIF = (wagasci_upstream_top, wagasci_upstream_side, wagasci_downstream_top, 
                         wagasci_downstream_side, wallmrd_north_top, wallmrd_north_bottom, wallmrd_south_top, 
                         wallmrd_south_bottom). Some plotters ignore the manually specified topology.
                        (default: All)'''))

    parser.add_argument('--randomize-channels', metavar='<num random channels>', dest='randomize_channels', type=int,
                        help='Plot a number of random channels', default=0, required=False)

    parser.add_argument('--dif', metavar='<DIF>', dest='dif', type=int,
                        help='DIF ID for gain and dark noise history', default=-1, required=False)

    parser.add_argument('--chip', metavar='<chip>', dest='chip', type=int,
                        help='chip ID for gain and dark noise history', default=-1, required=False)

    parser.add_argument('--channel', metavar='<channel>', dest='channel', type=int,
                        help='channel ID for gain and dark noise history', default=-1, required=False)

    parser.add_argument('--only-good', dest='only_good', required=False, default=False,
                        action='store_true', help=textwrap.dedent('''\
                        Only select runs and detectors whose flag is set to good (good run or good data)'''))

    # Flatten all arguments
    parsed_args = parser.parse_args(args=args)
    for name, value in vars(parsed_args).items():
        if isinstance(value, list):
            setattr(parsed_args, name, ','.join(value))

    return parsed_args


def parse_plotting_topology(topology_str):
    # type: (Optional[str]) -> topol.Topology
    """
    Read a string containing the list of enabled detectors and return a Topology object
    :param topology_str: list of enabled detectors
    :return: Topology object
    """
    topology_str = inflection.underscore(topology_str)
    iterate_by_dif = True if not topology_str or any(i in topology_str for i in ['side', 'top', 'bottom']) else False
    topology = topol.Topology(iterate_by_dif=iterate_by_dif)
    if topology_str is not None:
        topology.disable_all()
        iterator_class = topol.DifIndex if iterate_by_dif else topol.DetectorIndex
        for dif_name in [iterator_class.get_name_snakecase(dif_id.value) for dif_id in iterator_class]:
            if dif_name in topology_str:
                getattr(topology, dif_name).enable()
    return topology



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Pintaudi Giorgio
#

# pylint: disable=broad-except

"""Unit tests for database creation"""

import os
import shutil
import subprocess
import unittest

import wagascianpy.database.wagascidb
from wagascianpy.utils.utils import silentremove, silentremovedirs
import wagascianpy.utils.environment


class TestCreateWagasciDatabase(unittest.TestCase):

    def setUp(self):
        self.current_directory = os.path.abspath('.')
        self.test_local_database = os.path.join(self.current_directory, 'test.db')
        self.test_local_simple_repository = os.path.join(self.current_directory, 'test_simple_repo')
        self.test_local_borg_repository = os.path.join(self.current_directory, 'test_borg_repo')
        env = wagascianpy.utils.environment.WagasciEnvironment()
        wagascianpy.database.wagascidb.WagasciDataBase.wagasci_libdir = env["WAGASCI_LIBDIR"]
        self.tearDown()

    def tearDown(self):
        silentremove(self.test_local_database)
        silentremovedirs(self.test_local_borg_repository)

    def test_simple_repo_local_database(self):
        """Create a database locally from a local simple backup repository

        """
        with wagascianpy.database.wagascidb.WagasciDataBase(db_location=self.test_local_database,
                                                            repo_location=self.test_local_simple_repository,
                                                            is_borg_repo=False, update_db=False, rebuild_db=True) as db:
            self.assertEqual(6, len(db.get_all()))

    def test_simple_repo_remote_database(self):
        """Create a database remotely from a local simple backup repository

        """
        with wagascianpy.database.wagascidb.WagasciDataBase(db_location="kekcc:test.db",
                                                            repo_location=self.test_local_simple_repository,
                                                            is_borg_repo=False, update_db=False, rebuild_db=True) as db:
            self.assertEqual(6, len(db.get_all()))

    def test_borg_repo_local_database(self):
        """Create a database locally from a local borg backup repository

        """
        subprocess.call([os.path.join(self.current_directory, "create_test_borg_repo.sh")])

        with wagascianpy.database.wagascidb.WagasciDataBase(db_location=self.test_local_database,
                                                            repo_location=self.test_local_borg_repository,
                                                            is_borg_repo=True, update_db=False, rebuild_db=True) as db:
            self.assertEqual(6, len(db.get_all()))


class TestUpdateWagasciDatabase(unittest.TestCase):

    def setUp(self):
        self.current_directory = os.path.abspath('.')

        self.test_run_name = "physics_run_2020-01-18_15-12-41_64"
        self.test_local_database = os.path.join(self.current_directory, 'test.db')
        self.test_local_simple_repository = os.path.join(self.current_directory, 'test_simple_repo')
        self.test_local_borg_repository = os.path.join(self.current_directory, 'test_borg_repo')
        self.test_new_run_path = os.path.join(self.current_directory, self.test_run_name)
        self.test_old_run_path = os.path.join(self.test_local_simple_repository, self.test_run_name)

        env = wagascianpy.utils.environment.WagasciEnvironment()
        wagascianpy.database.wagascidb.WagasciDataBase.wagasci_libdir = env["WAGASCI_LIBDIR"]

        silentremovedirs(self.test_new_run_path)
        silentremove(self.test_local_database)
        silentremovedirs(self.test_local_borg_repository)
        shutil.move(self.test_old_run_path, self.test_new_run_path)

        subprocess.call([os.path.join(self.current_directory, "create_test_borg_repo.sh")])

    def tearDown(self):
        silentremove(self.test_local_database)
        silentremovedirs(self.test_local_borg_repository)

    def test_simple_repo_database_update(self):
        """Update a database locally from a local simple backup repository

        """
        with wagascianpy.database.wagascidb.WagasciDataBase(db_location=self.test_local_database,
                                                            repo_location=self.test_local_simple_repository,
                                                            is_borg_repo=False, update_db=False, rebuild_db=True) as db:
            self.assertEqual(5, len(db.get_all()))

        shutil.move(self.test_new_run_path, self.test_old_run_path)

        with wagascianpy.database.wagascidb.WagasciDataBase(db_location=self.test_local_database,
                                                            repo_location=self.test_local_simple_repository,
                                                            is_borg_repo=False, update_db=True, rebuild_db=False) as db:
            self.assertEqual(6, len(db.get_all()))

    def test_borg_repo_database_update(self):
        """Update a database locally from a local borg backup repository

        """
        with wagascianpy.database.wagascidb.WagasciDataBase(db_location=self.test_local_database,
                                                            repo_location=self.test_local_borg_repository,
                                                            is_borg_repo=True, update_db=False, rebuild_db=True) as db:
            self.assertEqual(5, len(db.get_all()))

        shutil.move(self.test_new_run_path, self.test_old_run_path)
        subprocess.call([os.path.join(self.current_directory, "create_test_borg_repo.sh")])

        with wagascianpy.database.wagascidb.WagasciDataBase(db_location=self.test_local_database,
                                                            repo_location=self.test_local_borg_repository,
                                                            is_borg_repo=True, update_db=True, rebuild_db=False) as db:
            self.assertEqual(6, len(db.get_all()))


if __name__ == '__main__':
    unittest.main()
